@@ -52,18 +52,26 @@ def quad(func, a, b, args=(), full_output=0, epsabs=1.49e-8, epsrel=1.49e-8,
 
     Parameters
     ----------
-    func : function
+    func : {function, scipy.LowLevelCallable}
         A Python function or method to integrate.  If `func` takes many
         arguments, it is integrated along the axis corresponding to the
         first argument.
-        If the user desires improved integration performance, then f may
-        instead be a ``ctypes`` function of the form:
 
-            f(int n, double args[n]),
+        If the user desires improved integration performance, then `f` may
+        be a `scipy.LowLevelCallable` with one of the signatures::
 
-        where ``args`` is an array of function arguments and ``n`` is the
-        length of ``args``. ``f.argtypes`` should be set to
-        ``(c_int, c_double)``, and ``f.restype`` should be ``(c_double,)``.
+            double func(double x)
+            double func(double x, void *user_data)
+            double func(int n, double *xx)
+            double func(int n, double *xx, void *user_data)
+
+        The ``user_data`` is the data contained in the `scipy.LowLevelCallable`.
+        In the call forms with ``xx``,  ``n`` is the length of the ``xx``
+        array which contains ``xx[0] == x`` and the rest of the items are
+        numbers contained in the ``args`` argument of quad.
+
+        In addition, certain ctypes call signatures are supported for
+        backward compatibility, but those should not be used in new code.
     a : float
         Lower limit of integration (use -numpy.inf for -infinity).
     b : float
@@ -307,16 +315,37 @@ def quad(func, a, b, args=(), full_output=0, epsabs=1.49e-8, epsrel=1.49e-8,
        print((1.0**3/3.0 + 1.0) - (0.0**3/3.0 + 0.0)) #Analytic result
        # 1.3333333333333333
 
+    Be aware that pulse shapes and other sharp features as compared to the
+    size of the integration interval may not be integrated correctly using
+    this method. A simplified example of this limitation is integrating a
+    y-axis reflected step function with many zero values within the integrals
+    bounds.
+
+    >>> y = lambda x: 1 if x<=0 else 0
+    >>> integrate.quad(y, -1, 1)
+    (1.0, 1.1102230246251565e-14)
+    >>> integrate.quad(y, -1, 100)
+    (1.0000000002199108, 1.0189464580163188e-08)
+    >>> integrate.quad(y, -1, 10000)
+    (0.0, 0.0)
+
     """
     if not isinstance(args, tuple):
         args = (args,)
-    if (weight is None):
+
+    # check the limits of integration: \int_a^b, expect a < b
+    flip, a, b = b < a, min(a, b), max(a, b)
+
+    if weight is None:
         retval = _quad(func, a, b, args, full_output, epsabs, epsrel, limit,
                        points)
     else:
         retval = _quad_weight(func, a, b, args, full_output, epsabs, epsrel,
                               limlst, limit, maxp1, weight, wvar, wopts)
 
+    if flip:
+        retval = (-retval[0],) + retval[1:]
+
     ier = retval[-1]
     if ier == 0:
         return retval[:-1]
@@ -348,15 +377,54 @@ def quad(func, a, b, args=(), full_output=0, epsabs=1.49e-8, epsrel=1.49e-8,
 
     if ier in [1,2,3,4,5,7]:
         if full_output:
-            if weight in ['cos','sin'] and (b == Inf or a == Inf):
+            if weight in ['cos', 'sin'] and (b == Inf or a == Inf):
                 return retval[:-1] + (msg, explain)
             else:
                 return retval[:-1] + (msg,)
         else:
             warnings.warn(msg, IntegrationWarning)
             return retval[:-1]
-    else:
-        raise ValueError(msg)
+
+    elif ier == 6:  # Forensic decision tree when QUADPACK throws ier=6
+        if epsabs <= 0:  # Small error tolerance - applies to all methods
+            if epsrel < max(50 * sys.float_info.epsilon, 5e-29):
+                msg = ("If 'errabs'<=0, 'epsrel' must be greater than both"
+                       " 5e-29 and 50*(machine epsilon).")
+            elif weight in ['sin', 'cos'] and (abs(a) + abs(b) == Inf):
+                msg = ("Sine or cosine weighted intergals with infinite domain"
+                       " must have 'epsabs'>0.")
+
+        elif weight is None:
+            if points is None:  # QAGSE/QAGIE
+                msg = ("Invalid 'limit' argument. There must be"
+                       " at least one subinterval")
+            else:  # QAGPE
+                if not (min(a, b) <= min(points) <= max(points) <= max(a, b)):
+                    msg = ("All break points in 'points' must lie within the"
+                           " integration limits.")
+                elif len(points) >= limit:
+                    msg = ("Number of break points ({:d})"
+                           " must be less than subinterval"
+                           " limit ({:d})").format(len(points), limit)
+
+        else:
+            if maxp1 < 1:
+                msg = "Chebyshev moment limit maxp1 must be >=1."
+
+            elif weight in ('cos', 'sin') and abs(a+b) == Inf:  # QAWFE
+                msg = "Cycle limit limlst must be >=3."
+
+            elif weight.startswith('alg'):  # QAWSE
+                if min(wvar) < -1:
+                    msg = "wvar parameters (alpha, beta) must both be >= -1."
+                if b < a:
+                    msg = "Integration limits a, b must satistfy a<b."
+
+            elif weight == 'cauchy' and wvar in (a, b):
+                msg = ("Parameter 'wvar' must not equal"
+                       " integration limits 'a' or 'b'.")
+
+    raise ValueError(msg)
 
 
 def _quad(func,a,b,args,full_output,epsabs,epsrel,limit,points):
@@ -384,14 +452,15 @@ def _quad(func,a,b,args,full_output,epsabs,epsrel,limit,points):
         if infbounds != 0:
             raise ValueError("Infinity inputs cannot be used with break points.")
         else:
-            nl = len(points)
-            the_points = numpy.zeros((nl+2,), float)
-            the_points[:nl] = points
+            #Duplicates force function evaluation at sinular points
+            the_points = numpy.unique(points)
+            the_points = the_points[a < the_points]
+            the_points = the_points[the_points < b]
+            the_points = numpy.concatenate((the_points, (0., 0.)))
             return _quadpack._qagpe(func,a,b,the_points,args,full_output,epsabs,epsrel,limit)
 
 
 def _quad_weight(func,a,b,args,full_output,epsabs,epsrel,limlst,limit,maxp1,weight,wvar,wopts):
-
     if weight not in ['cos','sin','alg','alg-loga','alg-logb','alg-log','cauchy']:
         raise ValueError("%s not a recognized weighting function." % weight)
 
@@ -434,7 +503,7 @@ def _quad_weight(func,a,b,args,full_output,epsabs,epsrel,limlst,limit,maxp1,weig
         if a in [-Inf,Inf] or b in [-Inf,Inf]:
             raise ValueError("Cannot integrate with this weight over an infinite interval.")
 
-        if weight[:3] == 'alg':
+        if weight.startswith('alg'):
             integr = strdict[weight]
             return _quadpack._qawse(func, a, b, wvar, integr, args,
                                     full_output, epsabs, epsrel, limit)
@@ -457,11 +526,11 @@ def dblquad(func, a, b, gfun, hfun, args=(), epsabs=1.49e-8, epsrel=1.49e-8):
         first argument and x the second argument.
     a, b : float
         The limits of integration in x: `a` < `b`
-    gfun : callable
+    gfun : callable or float
         The lower boundary curve in y which is a function taking a single
-        floating point argument (x) and returning a floating point result: a
-        lambda function can be useful here.
-    hfun : callable
+        floating point argument (x) and returning a floating point result
+        or a float indicating a constant boundary curve.
+    hfun : callable or float
         The upper boundary curve in y (same requirements as `gfun`).
     args : sequence, optional
         Extra arguments to pass to `func`.
@@ -491,10 +560,25 @@ def dblquad(func, a, b, gfun, hfun, args=(), epsabs=1.49e-8, epsrel=1.49e-8):
     romb : integrator for sampled data
     scipy.special : for coefficients and roots of orthogonal polynomials
 
+    Examples
+    --------
+
+    Compute the double integral of ``x * y**2`` over the box
+    ``x`` ranging from 0 to 2 and ``y`` ranging from 0 to 1.
+
+    >>> from scipy import integrate
+    >>> f = lambda y, x: x*y**2
+    >>> integrate.dblquad(f, 0, 2, lambda x: 0, lambda x: 1)
+        (0.6666666666666667, 7.401486830834377e-15)
+
     """
+    
     def temp_ranges(*args):
-        return [gfun(args[0]), hfun(args[0])]
-    return nquad(func, [temp_ranges, [a, b]], args=args)
+        return [gfun(args[0]) if callable(gfun) else gfun,
+                hfun(args[0]) if callable(hfun) else hfun]
+        
+    return nquad(func, [temp_ranges, [a, b]], args=args,
+            opts={"epsabs": epsabs, "epsrel": epsrel})
 
 
 def tplquad(func, a, b, gfun, hfun, qfun, rfun, args=(), epsabs=1.49e-8,
@@ -512,16 +596,17 @@ def tplquad(func, a, b, gfun, hfun, qfun, rfun, args=(), epsabs=1.49e-8,
         order (z, y, x).
     a, b : float
         The limits of integration in x: `a` < `b`
-    gfun : function
+    gfun : function or float
         The lower boundary curve in y which is a function taking a single
-        floating point argument (x) and returning a floating point result:
-        a lambda function can be useful here.
-    hfun : function
+        floating point argument (x) and returning a floating point result
+        or a float indicating a constant boundary curve.
+    hfun : function or float
         The upper boundary curve in y (same requirements as `gfun`).
-    qfun : function
+    qfun : function or float
         The lower boundary surface in z.  It must be a function that takes
-        two floats in the order (x, y) and returns a float.
-    rfun : function
+        two floats in the order (x, y) and returns a float or a float
+        indicating a constant boundary surface.
+    rfun : function or float
         The upper boundary surface in z. (Same requirements as `qfun`.)
     args : tuple, optional
         Extra arguments to pass to `func`.
@@ -551,6 +636,19 @@ def tplquad(func, a, b, gfun, hfun, qfun, rfun, args=(), epsabs=1.49e-8,
     odeint: ODE integrators
     scipy.special: For coefficients and roots of orthogonal polynomials
 
+    Examples
+    --------
+
+    Compute the triple integral of ``x * y * z``, over ``x`` ranging 
+    from 1 to 2, ``y`` ranging from 2 to 3, ``z`` ranging from 0 to 1.
+
+    >>> from scipy import integrate
+    >>> f = lambda z, y, x: x*y*z
+    >>> integrate.tplquad(f, 1, 2, lambda x: 2, lambda x: 3,
+    ...                   lambda x, y: 0, lambda x, y: 1)
+    (1.8750000000000002, 3.324644794257407e-14)
+
+
     """
     # f(z, y, x)
     # qfun/rfun (x, y)
@@ -560,13 +658,16 @@ def tplquad(func, a, b, gfun, hfun, qfun, rfun, args=(), epsabs=1.49e-8,
     # Stupid different API...
 
     def ranges0(*args):
-        return [qfun(args[1], args[0]), rfun(args[1], args[0])]
+        return [qfun(args[1], args[0]) if callable(qfun) else qfun,
+                rfun(args[1], args[0]) if callable(rfun) else rfun]
 
     def ranges1(*args):
-        return [gfun(args[0]), hfun(args[0])]
+        return [gfun(args[0]) if callable(gfun) else gfun,
+                hfun(args[0]) if callable(hfun) else hfun]
 
     ranges = [ranges0, ranges1, [a, b]]
-    return nquad(func, ranges, args=args)
+    return nquad(func, ranges, args=args,
+            opts={"epsabs": epsabs, "epsrel": epsrel})
 
 
 def nquad(func, ranges, args=None, opts=None, full_output=False):
@@ -580,33 +681,30 @@ def nquad(func, ranges, args=None, opts=None, full_output=False):
 
     Parameters
     ----------
-    func : callable
+    func : {callable, scipy.LowLevelCallable}
         The function to be integrated. Has arguments of ``x0, ... xn``,
         ``t0, tm``, where integration is carried out over ``x0, ... xn``, which
         must be floats.  Function signature should be
         ``func(x0, x1, ..., xn, t0, t1, ..., tm)``.  Integration is carried out
         in order.  That is, integration over ``x0`` is the innermost integral,
         and ``xn`` is the outermost.
-        If performance is a concern, this function may be a ctypes function of
-        the form::
 
-            f(int n, double args[n])
+        If the user desires improved integration performance, then `f` may
+        be a `scipy.LowLevelCallable` with one of the signatures::
+
+            double func(int n, double *xx)
+            double func(int n, double *xx, void *user_data)
 
         where ``n`` is the number of extra parameters and args is an array
-        of doubles of the additional parameters.  This function may then
-        be compiled to a dynamic/shared library then imported through
-        ``ctypes``, setting the function's argtypes to ``(c_int, c_double)``,
-        and the function's restype to ``(c_double)``.  Its pointer may then be
-        passed into `nquad` normally.
-        This allows the underlying Fortran library to evaluate the function in
-        the innermost integration calls without callbacks to Python, and also
-        speeds up the evaluation of the function itself.
+        of doubles of the additional parameters, the ``xx`` array contains the
+        coordinates. The ``user_data`` is the data contained in the
+        `scipy.LowLevelCallable`.
     ranges : iterable object
         Each element of ranges may be either a sequence  of 2 numbers, or else
         a callable that returns such a sequence.  ``ranges[0]`` corresponds to
         integration over x0, and so on.  If an element of ranges is a callable,
         then it will be called with all of the integration arguments available,
-        as well as any parametric arguments. e.g. if 
+        as well as any parametric arguments. e.g. if
         ``func = f(x0, x1, x2, t0, t1)``, then ``ranges[0]`` may be defined as
         either ``(a, b)`` or else as ``(a, b) = range0(x1, x2, t0, t1)``.
     args : iterable object, optional
@@ -618,7 +716,7 @@ def nquad(func, ranges, args=None, opts=None, full_output=False):
         default options from scipy.integrate.quad are used.  If a dict, the same
         options are used for all levels of integraion.  If a sequence, then each
         element of the sequence corresponds to a particular integration. e.g.
-        opts[0] corresponds to integration over x0, and so on. If a callable, 
+        opts[0] corresponds to integration over x0, and so on. If a callable,
         the signature must be the same as for ``ranges``. The available
         options together with their default values are:
 
@@ -633,8 +731,8 @@ def nquad(func, ranges, args=None, opts=None, full_output=False):
         For more information on these options, see `quad` and `quad_explain`.
 
     full_output : bool, optional
-        Partial implementation of ``full_output`` from scipy.integrate.quad. 
-        The number of integrand function evaluations ``neval`` can be obtained 
+        Partial implementation of ``full_output`` from scipy.integrate.quad.
+        The number of integrand function evaluations ``neval`` can be obtained
         by setting ``full_output=True`` when calling nquad.
 
     Returns
@@ -645,7 +743,7 @@ def nquad(func, ranges, args=None, opts=None, full_output=False):
         The maximum of the estimates of the absolute error in the various
         integration results.
     out_dict : dict, optional
-        A dict containing additional information on the integration. 
+        A dict containing additional information on the integration.
 
     See Also
     --------
