@@ -34,6 +34,8 @@ import math
 import numpy
 from . import _ni_support
 from . import _nd_image
+from . import _ni_docstrings
+from functools import wraps
 
 import warnings
 
@@ -41,22 +43,17 @@ __all__ = ['spline_filter1d', 'spline_filter', 'geometric_transform',
            'map_coordinates', 'affine_transform', 'shift', 'zoom', 'rotate']
 
 
-def _extend_mode_to_code(mode):
-    mode = _ni_support._extend_mode_to_code(mode)
-    return mode
-
-
+@_ni_docstrings.docfiller
 def spline_filter1d(input, order=3, axis=-1, output=numpy.float64):
     """
-    Calculates a one-dimensional spline filter along the given axis.
+    Calculate a one-dimensional spline filter along the given axis.
 
     The lines of the array along the given axis are filtered by a
     spline filter. The order of the spline must be >= 2 and <= 5.
 
     Parameters
     ----------
-    input : array_like
-        The input array.
+    %(input)s
     order : int, optional
         The order of the spline, default is 3.
     axis : int, optional
@@ -68,9 +65,8 @@ def spline_filter1d(input, order=3, axis=-1, output=numpy.float64):
 
     Returns
     -------
-    spline_filter1d : ndarray or None
-        The filtered input. If `output` is given as a parameter, None is
-        returned.
+    spline_filter1d : ndarray
+        The filtered input.
 
     """
     if order < 0 or order > 5:
@@ -78,13 +74,13 @@ def spline_filter1d(input, order=3, axis=-1, output=numpy.float64):
     input = numpy.asarray(input)
     if numpy.iscomplexobj(input):
         raise TypeError('Complex type not supported')
-    output, return_value = _ni_support._get_output(output, input)
+    output = _ni_support._get_output(output, input)
     if order in [0, 1]:
         output[...] = numpy.array(input)
     else:
         axis = _ni_support._check_axis(axis, input.ndim)
         _nd_image.spline_filter1d(input, order, axis, output)
-    return return_value
+    return output
 
 
 def spline_filter(input, order=3, output=numpy.float64):
@@ -111,32 +107,17 @@ def spline_filter(input, order=3, output=numpy.float64):
     input = numpy.asarray(input)
     if numpy.iscomplexobj(input):
         raise TypeError('Complex type not supported')
-    output, return_value = _ni_support._get_output(output, input)
+    output = _ni_support._get_output(output, input)
     if order not in [0, 1] and input.ndim > 0:
         for axis in range(input.ndim):
             spline_filter1d(input, order, axis, output=output)
             input = output
     else:
         output[...] = input[...]
-    return return_value
-
-
-def _geometric_transform(input, mapping, coordinates, matrix, offset, output,
-                         order, mode, cval, extra_arguments, extra_keywords):
-    """
-    Wrapper around _nd_image.geometric_transform to work around
-    endianness issues
-    """
-    _nd_image.geometric_transform(
-        input, mapping, coordinates, matrix, offset, output,
-        order, mode, cval, extra_arguments, extra_keywords)
-
-    if output is not None and not output.dtype.isnative:
-        output.byteswap(True)
-
     return output
 
 
+@_ni_docstrings.docfiller
 def geometric_transform(input, mapping, output_shape=None,
                         output=None, order=3,
                         mode='constant', cval=0.0, prefilter=True,
@@ -151,32 +132,20 @@ def geometric_transform(input, mapping, output_shape=None,
 
     Parameters
     ----------
-    input : array_like
-        The input array.
+    %(input)s
     mapping : {callable, scipy.LowLevelCallable}
         A callable object that accepts a tuple of length equal to the output
         array rank, and returns the corresponding input coordinates as a tuple
         of length equal to the input array rank.
     output_shape : tuple of ints, optional
         Shape tuple.
-    output : ndarray or dtype, optional
-        The array in which to place the output, or the dtype of the returned
-        array.
+    %(output)s
     order : int, optional
         The order of the spline interpolation, default is 3.
         The order has to be in the range 0-5.
-    mode : str, optional
-        Points outside the boundaries of the input are filled according
-        to the given mode ('constant', 'nearest', 'reflect', 'mirror' or 'wrap').
-        Default is 'constant'.
-    cval : scalar, optional
-        Value used for points outside the boundaries of the input if
-        ``mode='constant'``. Default is 0.0
-    prefilter : bool, optional
-        The parameter prefilter determines if the input is pre-filtered with
-        `spline_filter` before interpolation (necessary for spline
-        interpolation of order > 1).  If False, it is assumed that the input is
-        already filtered. Default is True.
+    %(mode)s
+    %(cval)s
+    %(prefilter)s
     extra_arguments : tuple, optional
         Extra arguments passed to `mapping`.
     extra_keywords : dict, optional
@@ -184,9 +153,8 @@ def geometric_transform(input, mapping, output_shape=None,
 
     Returns
     -------
-    return_value : ndarray or None
-        The filtered input. If `output` is given as a parameter, None is
-        returned.
+    output : ndarray
+        The filtered input.
 
     See Also
     --------
@@ -200,9 +168,9 @@ def geometric_transform(input, mapping, output_shape=None,
 
     .. code:: c
 
-       int mapping(npy_intp *output_coordinates, double *input_coordinates, 
+       int mapping(npy_intp *output_coordinates, double *input_coordinates,
                    int output_rank, int input_rank, void *user_data)
-       int mapping(intptr_t *output_coordinates, double *input_coordinates, 
+       int mapping(intptr_t *output_coordinates, double *input_coordinates,
                    int output_rank, int input_rank, void *user_data)
 
     The calling function iterates over the elements of the output array,
@@ -211,32 +179,46 @@ def geometric_transform(input, mapping, output_shape=None,
     callback function must return the coordinates at which the input must
     be interpolated in ``input_coordinates``. The rank of the input and
     output arrays are given by ``input_rank`` and ``output_rank``
-    respectively.  ``user_data`` is the data pointer provided 
-    to `scipy.LowLevelCallable` as-is. 
+    respectively.  ``user_data`` is the data pointer provided
+    to `scipy.LowLevelCallable` as-is.
 
-    The callback function must return an integer error status that is zero 
+    The callback function must return an integer error status that is zero
     if something went wrong and one otherwise. If an error occurs, you should
     normally set the python error status with an informative message
     before returning, otherwise a default error message is set by the
     calling function.
 
-    In addition, some other low-level function pointer specifications 
+    In addition, some other low-level function pointer specifications
     are accepted, but these are for backward compatibility only and should
     not be used in new code.
 
     Examples
     --------
-    >>> from scipy import ndimage
+    >>> import numpy as np
+    >>> from scipy.ndimage import geometric_transform
     >>> a = np.arange(12.).reshape((4, 3))
     >>> def shift_func(output_coords):
     ...     return (output_coords[0] - 0.5, output_coords[1] - 0.5)
     ...
-    >>> ndimage.geometric_transform(a, shift_func)
+    >>> geometric_transform(a, shift_func)
     array([[ 0.   ,  0.   ,  0.   ],
            [ 0.   ,  1.362,  2.738],
            [ 0.   ,  4.812,  6.187],
            [ 0.   ,  8.263,  9.637]])
 
+    >>> b = [1, 2, 3, 4, 5]
+    >>> def shift_func(output_coords):
+    ...     return (output_coords[0] - 3,)
+    ...
+    >>> geometric_transform(b, shift_func, mode='constant')
+    array([0, 0, 0, 1, 2])
+    >>> geometric_transform(b, shift_func, mode='nearest')
+    array([1, 1, 1, 1, 2])
+    >>> geometric_transform(b, shift_func, mode='reflect')
+    array([3, 2, 1, 1, 2])
+    >>> geometric_transform(b, shift_func, mode='wrap')
+    array([2, 3, 4, 1, 2])
+
     """
     if order < 0 or order > 5:
         raise RuntimeError('spline order not supported')
@@ -247,18 +229,19 @@ def geometric_transform(input, mapping, output_shape=None,
         output_shape = input.shape
     if input.ndim < 1 or len(output_shape) < 1:
         raise RuntimeError('input and output rank must be > 0')
-    mode = _extend_mode_to_code(mode)
+    mode = _ni_support._extend_mode_to_code(mode)
     if prefilter and order > 1:
         filtered = spline_filter(input, order, output=numpy.float64)
     else:
         filtered = input
-    output, return_value = _ni_support._get_output(output, input,
-                                                   shape=output_shape)
-    _geometric_transform(filtered, mapping, None, None, None, output,
-                         order, mode, cval, extra_arguments, extra_keywords)
-    return return_value
+    output = _ni_support._get_output(output, input, shape=output_shape)
+    _nd_image.geometric_transform(filtered, mapping, None, None, None, output,
+                                  order, mode, cval, extra_arguments,
+                                  extra_keywords)
+    return output
 
 
+@_ni_docstrings.docfiller
 def map_coordinates(input, coordinates, output=None, order=3,
                     mode='constant', cval=0.0, prefilter=True):
     """
@@ -276,28 +259,16 @@ def map_coordinates(input, coordinates, output=None, order=3,
 
     Parameters
     ----------
-    input : ndarray
-        The input array.
+    %(input)s
     coordinates : array_like
         The coordinates at which `input` is evaluated.
-    output : ndarray or dtype, optional
-        The array in which to place the output, or the dtype of the returned
-        array.
+    %(output)s
     order : int, optional
         The order of the spline interpolation, default is 3.
         The order has to be in the range 0-5.
-    mode : str, optional
-        Points outside the boundaries of the input are filled according
-        to the given mode ('constant', 'nearest', 'reflect', 'mirror' or 'wrap').
-        Default is 'constant'.
-    cval : scalar, optional
-        Value used for points outside the boundaries of the input if
-        ``mode='constant'``. Default is 0.0
-    prefilter : bool, optional
-        The parameter prefilter determines if the input is pre-filtered with
-        `spline_filter` before interpolation (necessary for spline
-        interpolation of order > 1).  If False, it is assumed that the input is
-        already filtered. Default is True.
+    %(mode)s
+    %(cval)s
+    %(prefilter)s
 
     Returns
     -------
@@ -346,24 +317,72 @@ def map_coordinates(input, coordinates, output=None, order=3,
         raise RuntimeError('input and output rank must be > 0')
     if coordinates.shape[0] != input.ndim:
         raise RuntimeError('invalid shape for coordinate array')
-    mode = _extend_mode_to_code(mode)
+    mode = _ni_support._extend_mode_to_code(mode)
     if prefilter and order > 1:
         filtered = spline_filter(input, order, output=numpy.float64)
     else:
         filtered = input
-    output, return_value = _ni_support._get_output(output, input,
+    output = _ni_support._get_output(output, input,
                                                    shape=output_shape)
-    _geometric_transform(filtered, None, coordinates, None, None,
-                         output, order, mode, cval, None, None)
-    return return_value
+    _nd_image.geometric_transform(filtered, None, coordinates, None, None,
+                                  output, order, mode, cval, None, None)
+    return output
 
 
+@_ni_docstrings.docfiller
 def affine_transform(input, matrix, offset=0.0, output_shape=None,
                      output=None, order=3,
                      mode='constant', cval=0.0, prefilter=True):
     """
     Apply an affine transformation.
 
+    Given an output image pixel index vector ``o``, the pixel value
+    is determined from the input image at position
+    ``np.dot(matrix, o) + offset``.
+
+    Parameters
+    ----------
+    %(input)s
+    matrix : ndarray
+        The inverse coordinate transformation matrix, mapping output
+        coordinates to input coordinates. If ``ndim`` is the number of
+        dimensions of ``input``, the given matrix must have one of the
+        following shapes:
+
+            - ``(ndim, ndim)``: the linear transformation matrix for each
+              output coordinate.
+            - ``(ndim,)``: assume that the 2D transformation matrix is
+              diagonal, with the diagonal specified by the given value. A more
+              efficient algorithm is then used that exploits the separability
+              of the problem.
+            - ``(ndim + 1, ndim + 1)``: assume that the transformation is
+              specified using homogeneous coordinates [1]_. In this case, any
+              value passed to ``offset`` is ignored.
+            - ``(ndim, ndim + 1)``: as above, but the bottom row of a
+              homogeneous transformation matrix is always ``[0, 0, ..., 1]``,
+              and may be omitted.
+
+    offset : float or sequence, optional
+        The offset into the array where the transform is applied. If a float,
+        `offset` is the same for each axis. If a sequence, `offset` should
+        contain one value for each axis.
+    output_shape : tuple of ints, optional
+        Shape tuple.
+    %(output)s
+    order : int, optional
+        The order of the spline interpolation, default is 3.
+        The order has to be in the range 0-5.
+    %(mode)s
+    %(cval)s
+    %(prefilter)s
+
+    Returns
+    -------
+    affine_transform : ndarray
+        The transformed input.
+
+    Notes
+    -----
     The given matrix and offset are used to find for each point in the
     output the corresponding coordinates in the input by an affine
     transformation. The value of the input at those coordinates is
@@ -371,60 +390,17 @@ def affine_transform(input, matrix, offset=0.0, output_shape=None,
     outside the boundaries of the input are filled according to the given
     mode.
 
-    Given an output image pixel index vector ``o``, the pixel value
-    is determined from the input image at position ``np.dot(matrix,o) + offset``.
-
-    A diagonal matrix can be specified by supplying a one-dimensional
-    array-like to the matrix parameter, in which case a more efficient
-    algorithm is applied.
-
     .. versionchanged:: 0.18.0
         Previously, the exact interpretation of the affine transformation
         depended on whether the matrix was supplied as a one-dimensional or
         two-dimensional array. If a one-dimensional array was supplied
         to the matrix parameter, the output pixel value at index ``o``
-        was determined from the input image at position ``matrix * (o + offset)``.
+        was determined from the input image at position
+        ``matrix * (o + offset)``.
 
-    Parameters
+    References
     ----------
-    input : ndarray
-        The input array.
-    matrix : ndarray
-        The matrix must be two-dimensional or can also be given as a
-        one-dimensional sequence or array. In the latter case, it is assumed
-        that the matrix is diagonal. A more efficient algorithms is then
-        applied that exploits the separability of the problem.
-    offset : float or sequence, optional
-        The offset into the array where the transform is applied. If a float,
-        `offset` is the same for each axis. If a sequence, `offset` should
-        contain one value for each axis.
-    output_shape : tuple of ints, optional
-        Shape tuple.
-    output : ndarray or dtype, optional
-        The array in which to place the output, or the dtype of the returned
-        array.
-    order : int, optional
-        The order of the spline interpolation, default is 3.
-        The order has to be in the range 0-5.
-    mode : str, optional
-        Points outside the boundaries of the input are filled according
-        to the given mode ('constant', 'nearest', 'reflect', 'mirror' or 'wrap').
-        Default is 'constant'.
-    cval : scalar, optional
-        Value used for points outside the boundaries of the input if
-        ``mode='constant'``. Default is 0.0
-    prefilter : bool, optional
-        The parameter prefilter determines if the input is pre-filtered with
-        `spline_filter` before interpolation (necessary for spline
-        interpolation of order > 1).  If False, it is assumed that the input is
-        already filtered. Default is True.
-
-    Returns
-    -------
-    affine_transform : ndarray or None
-        The transformed input. If `output` is given as a parameter, None is
-        returned.
-
+    .. [1] https://en.wikipedia.org/wiki/Homogeneous_coordinates
     """
     if order < 0 or order > 5:
         raise RuntimeError('spline order not supported')
@@ -435,16 +411,28 @@ def affine_transform(input, matrix, offset=0.0, output_shape=None,
         output_shape = input.shape
     if input.ndim < 1 or len(output_shape) < 1:
         raise RuntimeError('input and output rank must be > 0')
-    mode = _extend_mode_to_code(mode)
+    mode = _ni_support._extend_mode_to_code(mode)
     if prefilter and order > 1:
         filtered = spline_filter(input, order, output=numpy.float64)
     else:
         filtered = input
-    output, return_value = _ni_support._get_output(output, input,
+    output = _ni_support._get_output(output, input,
                                                    shape=output_shape)
     matrix = numpy.asarray(matrix, dtype=numpy.float64)
     if matrix.ndim not in [1, 2] or matrix.shape[0] < 1:
         raise RuntimeError('no proper affine matrix provided')
+    if (matrix.ndim == 2 and matrix.shape[1] == input.ndim + 1 and
+            (matrix.shape[0] in [input.ndim, input.ndim + 1])):
+        if matrix.shape[0] == input.ndim + 1:
+            exptd = [0] * input.ndim + [1]
+            if not numpy.all(matrix[input.ndim] == exptd):
+                msg = ('Expected homogeneous transformation matrix with '
+                       'shape %s for image shape %s, but bottom row was '
+                       'not equal to %s' % (matrix.shape, input.shape, exptd))
+                raise ValueError(msg)
+        # assume input is homogeneous coordinate transformation matrix
+        offset = matrix[:input.ndim, input.ndim]
+        matrix = matrix[:input.ndim, :input.ndim]
     if matrix.shape[0] != input.ndim:
         raise RuntimeError('affine matrix has wrong number of rows')
     if matrix.ndim == 2 and matrix.shape[1] != output.ndim:
@@ -466,11 +454,12 @@ def affine_transform(input, matrix, offset=0.0, output_shape=None,
         _nd_image.zoom_shift(filtered, matrix, offset/matrix, output, order,
                              mode, cval)
     else:
-        _geometric_transform(filtered, None, None, matrix, offset,
-                             output, order, mode, cval, None, None)
-    return return_value
+        _nd_image.geometric_transform(filtered, None, None, matrix, offset,
+                                      output, order, mode, cval, None, None)
+    return output
 
 
+@_ni_docstrings.docfiller
 def shift(input, shift, output=None, order=3, mode='constant', cval=0.0,
           prefilter=True):
     """
@@ -482,35 +471,22 @@ def shift(input, shift, output=None, order=3, mode='constant', cval=0.0,
 
     Parameters
     ----------
-    input : ndarray
-        The input array.
-    shift : float or sequence, optional
+    %(input)s
+    shift : float or sequence
         The shift along the axes. If a float, `shift` is the same for each
         axis. If a sequence, `shift` should contain one value for each axis.
-    output : ndarray or dtype, optional
-        The array in which to place the output, or the dtype of the returned
-        array.
+    %(output)s
     order : int, optional
         The order of the spline interpolation, default is 3.
         The order has to be in the range 0-5.
-    mode : str, optional
-        Points outside the boundaries of the input are filled according
-        to the given mode ('constant', 'nearest', 'reflect', 'mirror' or 'wrap').
-        Default is 'constant'.
-    cval : scalar, optional
-        Value used for points outside the boundaries of the input if
-        ``mode='constant'``. Default is 0.0
-    prefilter : bool, optional
-        The parameter prefilter determines if the input is pre-filtered with
-        `spline_filter` before interpolation (necessary for spline
-        interpolation of order > 1).  If False, it is assumed that the input is
-        already filtered. Default is True.
+    %(mode)s
+    %(cval)s
+    %(prefilter)s
 
     Returns
     -------
-    shift : ndarray or None
-        The shifted input. If `output` is given as a parameter, None is
-        returned.
+    shift : ndarray
+        The shifted input.
 
     """
     if order < 0 or order > 5:
@@ -520,21 +496,22 @@ def shift(input, shift, output=None, order=3, mode='constant', cval=0.0,
         raise TypeError('Complex type not supported')
     if input.ndim < 1:
         raise RuntimeError('input and output rank must be > 0')
-    mode = _extend_mode_to_code(mode)
+    mode = _ni_support._extend_mode_to_code(mode)
     if prefilter and order > 1:
         filtered = spline_filter(input, order, output=numpy.float64)
     else:
         filtered = input
-    output, return_value = _ni_support._get_output(output, input)
+    output = _ni_support._get_output(output, input)
     shift = _ni_support._normalize_sequence(shift, input.ndim)
     shift = [-ii for ii in shift]
     shift = numpy.asarray(shift, dtype=numpy.float64)
     if not shift.flags.contiguous:
         shift = shift.copy()
     _nd_image.zoom_shift(filtered, None, shift, output, order, mode, cval)
-    return return_value
+    return output
 
 
+@_ni_docstrings.docfiller
 def zoom(input, zoom, output=None, order=3, mode='constant', cval=0.0,
          prefilter=True):
     """
@@ -544,36 +521,42 @@ def zoom(input, zoom, output=None, order=3, mode='constant', cval=0.0,
 
     Parameters
     ----------
-    input : ndarray
-        The input array.
-    zoom : float or sequence, optional
+    %(input)s
+    zoom : float or sequence
         The zoom factor along the axes. If a float, `zoom` is the same for each
         axis. If a sequence, `zoom` should contain one value for each axis.
-    output : ndarray or dtype, optional
-        The array in which to place the output, or the dtype of the returned
-        array.
+    %(output)s
     order : int, optional
         The order of the spline interpolation, default is 3.
         The order has to be in the range 0-5.
-    mode : str, optional
-        Points outside the boundaries of the input are filled according
-        to the given mode ('constant', 'nearest', 'reflect', 'mirror' or 'wrap').
-        Default is 'constant'.
-    cval : scalar, optional
-        Value used for points outside the boundaries of the input if
-        ``mode='constant'``. Default is 0.0
-    prefilter : bool, optional
-        The parameter prefilter determines if the input is pre-filtered with
-        `spline_filter` before interpolation (necessary for spline
-        interpolation of order > 1).  If False, it is assumed that the input is
-        already filtered. Default is True.
+    %(mode)s
+    %(cval)s
+    %(prefilter)s
 
     Returns
     -------
-    zoom : ndarray or None
-        The zoomed input. If `output` is given as a parameter, None is
-        returned.
+    zoom : ndarray
+        The zoomed input.
 
+    Examples
+    --------
+    >>> from scipy import ndimage, misc
+    >>> import matplotlib.pyplot as plt
+
+    >>> fig = plt.figure()
+    >>> ax1 = fig.add_subplot(121)  # left side
+    >>> ax2 = fig.add_subplot(122)  # right side
+    >>> ascent = misc.ascent()
+    >>> result = ndimage.zoom(ascent, 3.0)
+    >>> ax1.imshow(ascent)
+    >>> ax2.imshow(result)
+    >>> plt.show()
+
+    >>> print(ascent.shape)
+    (512, 512)
+
+    >>> print(result.shape)
+    (1536, 1536)
     """
     if order < 0 or order > 5:
         raise RuntimeError('spline order not supported')
@@ -582,7 +565,7 @@ def zoom(input, zoom, output=None, order=3, mode='constant', cval=0.0,
         raise TypeError('Complex type not supported')
     if input.ndim < 1:
         raise RuntimeError('input and output rank must be > 0')
-    mode = _extend_mode_to_code(mode)
+    mode = _ni_support._extend_mode_to_code(mode)
     if prefilter and order > 1:
         filtered = spline_filter(input, order, output=numpy.float64)
     else:
@@ -606,11 +589,11 @@ def zoom(input, zoom, output=None, order=3, mode='constant', cval=0.0,
                         out=numpy.ones_like(input.shape, dtype=numpy.float64),
                         where=zoom_div != 0)
 
-    output, return_value = _ni_support._get_output(output, input,
+    output = _ni_support._get_output(output, input,
                                                    shape=output_shape)
     zoom = numpy.ascontiguousarray(zoom)
     _nd_image.zoom_shift(filtered, zoom, None, output, order, mode, cval)
-    return return_value
+    return output
 
 
 def _minmax(coor, minc, maxc):
@@ -625,8 +608,7 @@ def _minmax(coor, minc, maxc):
     return minc, maxc
 
 
-def rotate(input, angle, axes=(1, 0), reshape=True,
-           output=None, order=3,
+def rotate(input, angle, axes=(1, 0), reshape=True, output=None, order=3,
            mode='constant', cval=0.0, prefilter=True):
     """
     Rotate an array.
@@ -636,8 +618,7 @@ def rotate(input, angle, axes=(1, 0), reshape=True,
 
     Parameters
     ----------
-    input : ndarray
-        The input array.
+    %(input)s
     angle : float
         The rotation angle in degrees.
     axes : tuple of 2 ints, optional
@@ -646,30 +627,18 @@ def rotate(input, angle, axes=(1, 0), reshape=True,
     reshape : bool, optional
         If `reshape` is true, the output shape is adapted so that the input
         array is contained completely in the output. Default is True.
-    output : ndarray or dtype, optional
-        The array in which to place the output, or the dtype of the returned
-        array.
+    %(output)s
     order : int, optional
         The order of the spline interpolation, default is 3.
         The order has to be in the range 0-5.
-    mode : str, optional
-        Points outside the boundaries of the input are filled according
-        to the given mode ('constant', 'nearest', 'reflect', 'mirror' or 'wrap').
-        Default is 'constant'.
-    cval : scalar, optional
-        Value used for points outside the boundaries of the input if
-        ``mode='constant'``. Default is 0.0
-    prefilter : bool, optional
-        The parameter prefilter determines if the input is pre-filtered with
-        `spline_filter` before interpolation (necessary for spline
-        interpolation of order > 1).  If False, it is assumed that the input is
-        already filtered. Default is True.
+    %(mode)s
+    %(cval)s
+    %(prefilter)s
 
     Returns
     -------
-    rotate : ndarray or None
-        The rotated input. If `output` is given as a parameter, None is
-        returned.
+    rotate : ndarray
+        The rotated input.
 
     """
     input = numpy.asarray(input)
@@ -689,12 +658,12 @@ def rotate(input, angle, axes=(1, 0), reshape=True,
     m21 = -math.sin(angle)
     m22 = math.cos(angle)
     matrix = numpy.array([[m11, m12],
-                             [m21, m22]], dtype=numpy.float64)
+                          [m21, m22]], dtype=numpy.float64)
     iy = input.shape[axes[0]]
     ix = input.shape[axes[1]]
     if reshape:
         mtrx = numpy.array([[m11, -m21],
-                               [-m12, m22]], dtype=numpy.float64)
+                            [-m12, m22]], dtype=numpy.float64)
         minc = [0, 0]
         maxc = [0, 0]
         coor = numpy.dot(mtrx, [0, ix])
@@ -720,14 +689,14 @@ def rotate(input, angle, axes=(1, 0), reshape=True,
     output_shape[axes[0]] = oy
     output_shape[axes[1]] = ox
     output_shape = tuple(output_shape)
-    output, return_value = _ni_support._get_output(output, input,
+    output = _ni_support._get_output(output, input,
                                                    shape=output_shape)
     if input.ndim <= 2:
         affine_transform(input, matrix, offset, output_shape, output,
                          order, mode, cval, prefilter)
     else:
         coordinates = []
-        size = numpy.product(input.shape,axis=0)
+        size = numpy.product(input.shape, axis=0)
         size //= input.shape[axes[0]]
         size //= input.shape[axes[1]]
         for ii in range(input.ndim):
@@ -751,4 +720,4 @@ def rotate(input, angle, axes=(1, 0), reshape=True,
                     break
                 else:
                     coordinates[jj] = 0
-    return return_value
+    return output
