@@ -15,7 +15,7 @@ common FORTRAN code in the packages MINPACK, ODEPACK, and QUADPACK plus
 some differential algebraic equation solvers.
 
 The wrappers are meant to be nearly direct translations between the
-FORTAN code and Python.  Some parameters like sizes do not need to be 
+FORTRAN code and Python.  Some parameters like sizes do not need to be 
 passed since they are available from the objects.  
 
 It is anticipated that a pure Python module be written to call these lower
@@ -39,7 +39,6 @@ the result tuple when the full_output argument is non-zero.
 
 #define PYERR(errobj,message) {PyErr_SetString(errobj,message); goto fail;}
 #define PYERR2(errobj,message) {PyErr_Print(); PyErr_SetString(errobj, message); goto fail;}
-#define ISCONTIGUOUS(m) ((m)->flags & NPY_CONTIGUOUS)
 
 #define STORE_VARS() PyObject *store_multipack_globals[4]; int store_multipack_globals3;
 
@@ -91,13 +90,13 @@ the result tuple when the full_output argument is non-zero.
   if (o_diag == NULL || o_diag == Py_None) { \
     ap_diag = (PyArrayObject *)PyArray_SimpleNew(1,&n,NPY_DOUBLE); \
     if (ap_diag == NULL) goto fail; \
-    diag = (double *)ap_diag -> data; \
+    diag = (double *)PyArray_DATA(ap_diag); \
     mode = 1; \
   } \
   else { \
     ap_diag = (PyArrayObject *)PyArray_ContiguousFromObject(o_diag, NPY_DOUBLE, 1, 1); \
     if (ap_diag == NULL) goto fail; \
-    diag = (double *)ap_diag -> data; \
+    diag = (double *)PyArray_DATA(ap_diag); \
     mode = 2; } }
 
 #define MATRIXC2F(jac,data,m,n) {double *p1=(double *)(jac), *p2, *p3=(double *)(data);\
@@ -111,7 +110,7 @@ static PyObject *multipack_python_jacobian=NULL;
 static PyObject *multipack_extra_arguments=NULL;    /* a tuple */
 static int multipack_jac_transpose=1;
 
-static PyObject *call_python_function(PyObject *func, npy_intp n, double *x, PyObject *args, int dim, PyObject *error_obj)
+static PyObject *call_python_function(PyObject *func, npy_intp n, double *x, PyObject *args, int dim, PyObject *error_obj, npy_intp out_size)
 {
   /*
     This is a generic function to call a python function that takes a 1-D
@@ -131,6 +130,7 @@ static PyObject *call_python_function(PyObject *func, npy_intp n, double *x, PyO
   PyObject *arg1 = NULL;
   PyObject *result = NULL;
   PyArrayObject *result_array = NULL;
+  npy_intp fvec_sz = 0;
 
   /* Build sequence argument from inputs */
   sequence = (PyArrayObject *)PyArray_SimpleNewFromData(1, &n, NPY_DOUBLE, (char *)x);
@@ -159,6 +159,13 @@ static PyObject *call_python_function(PyObject *func, npy_intp n, double *x, PyO
   if ((result_array = (PyArrayObject *)PyArray_ContiguousFromObject(result, NPY_DOUBLE, dim-1, dim))==NULL) 
     PYERR2(error_obj,"Result from function call is not a proper array of floats.");
 
+  fvec_sz = PyArray_SIZE(result_array);
+  if(out_size != -1 && fvec_sz != out_size){
+      PyErr_SetString(PyExc_ValueError, "The array returned by a function changed size between calls");
+      Py_DECREF(result_array);
+      goto fail;
+  }
+
   Py_DECREF(result);
   Py_DECREF(arglist);
   return (PyObject *)result_array;
