@@ -852,6 +852,7 @@ PyObject *PyArray_OrderFilterND(PyObject *op1, PyObject *op2, int order) {
 	intp *ret_ind;
 	CompareFunction compare_func;
 	char *zptr=NULL;
+	PyArray_CopySwapFunc *copyswap;
 
 	/* Get Array objects from input */
 	typenum = PyArray_ObjectType(op1, 0);  
@@ -907,6 +908,8 @@ PyObject *PyArray_OrderFilterND(PyObject *op1, PyObject *op2, int order) {
 	os = PyArray_ITEMSIZE(ret);
 	op = PyArray_DATA(ret);
 
+	copyswap = PyArray_DESCR(ret)->f->copyswap;
+
 	bytes_in_array = PyArray_NDIM(ap1)*sizeof(intp);
 	mode_dep = malloc(bytes_in_array);
 	for (k = 0; k < PyArray_NDIM(ap1); k++) { 
@@ -980,8 +983,15 @@ PyObject *PyArray_OrderFilterND(PyObject *op1, PyObject *op2, int order) {
 
 	  fill_buffer(ap1_ptr,ap1,ap2,sort_buffer,n2,check,b_ind,temp_ind,offsets);
 	  qsort(sort_buffer, n2_nonzero, is1, compare_func);
-	  memcpy(op, sort_buffer + order*is1, os);
-          
+
+	  /*
+	   * Use copyswap for correct refcounting with object arrays
+	   * (sort_buffer has borrowed references, op owns references). Note
+	   * also that os == PyArray_ITEMSIZE(ret) and we are copying a single
+	   * scalar here.
+	   */
+	  copyswap(op, sort_buffer + order*is1, 0, NULL);
+
           /* increment index counter */
 	  incr = increment(ret_ind,PyArray_NDIM(ret),PyArray_DIMS(ret)); 
           /* increment to next output index */
@@ -1023,11 +1033,10 @@ static PyObject *sigtools_convolve2d(PyObject *NPY_UNUSED(dummy), PyObject *args
 
     PyObject *in1=NULL, *in2=NULL, *fill_value=NULL;
     int mode=2, boundary=0, typenum, flag, flip=1, ret;
-    intp *aout_dimens=NULL, *dims=NULL;
-    char zeros[32];  /* Zeros */
+    intp *aout_dimens=NULL;
     int i;
     PyArrayObject *ain1=NULL, *ain2=NULL, *aout=NULL;
-    PyArrayObject *afill=NULL, *newfill=NULL;
+    PyArrayObject *afill=NULL;
 
     if (!PyArg_ParseTuple(args, "OO|iiiO", &in1, &in2, &flip, &mode, &boundary,
                           &fill_value)) {
@@ -1043,21 +1052,45 @@ static PyObject *sigtools_convolve2d(PyObject *NPY_UNUSED(dummy), PyObject *args
 
     if ((boundary != PAD) && (boundary != REFLECT) && (boundary != CIRCULAR))
       PYERR("Incorrect boundary value.");
-    if (boundary == PAD) {
-	if (fill_value == NULL) {
-	    newfill = (PyArrayObject *)PyArray_SimpleNewFromData(0, dims, typenum,
-                                                                 zeros);
-	}
-	else {
-	    afill = (PyArrayObject *)PyArray_FromObject(fill_value, NPY_CDOUBLE, 0, 0);
-	    if (afill == NULL) goto fail;
-	    newfill = (PyArrayObject *)PyArray_Cast(afill, typenum);
-	}
-	if (newfill == NULL) goto fail;
+
+    if ((boundary == PAD) & (fill_value != NULL)) {
+        afill = (PyArrayObject *)PyArray_FromObject(fill_value, typenum, 0, 0);
+        if (afill == NULL) {
+            /* For backwards compatibility try go via complex */
+            PyArrayObject *tmp;
+            PyErr_Clear();
+            tmp = (PyArrayObject *)PyArray_FromObject(fill_value,
+                                                      NPY_CDOUBLE, 0, 0);
+            if (tmp == NULL) goto fail;
+            afill = (PyArrayObject *)PyArray_Cast(tmp, typenum);
+            Py_DECREF(tmp);
+            if (afill == NULL) goto fail;
+            /* Deprecated 2017-07, scipy version 1.0.0 */
+            if (DEPRECATE("could not cast `fillvalue` directly to the output "
+                          "type (it was first converted to complex). "
+                          "This is deprecated and will raise an error in the "
+                          "future.") < 0) {
+                goto fail;
+            }
+        }
+        if (PyArray_SIZE(afill) != 1) {
+            if (PyArray_SIZE(afill) == 0) {
+                PyErr_SetString(PyExc_ValueError,
+                                "`fillvalue` cannot be an empty array.");
+                goto fail;
+            }
+            /* Deprecated 2017-07, scipy version 1.0.0 */
+            if (DEPRECATE("`fillvalue` must be scalar or an array with "
+                          "one element. "
+                          "This will raise an error in the future.") < 0) {
+                goto fail;
+            }
+        }
     }
     else {
-	newfill = (PyArrayObject *)PyArray_SimpleNewFromData(0, dims, typenum, zeros);
-	if (newfill == NULL) goto fail;
+        /* Create a zero filled array */
+        afill = (PyArrayObject *)PyArray_ZEROS(0, NULL, typenum, 0);
+        if (afill == NULL) goto fail;
     }
 
     aout_dimens = malloc(PyArray_NDIM(ain1)*sizeof(intp));
@@ -1099,13 +1132,13 @@ static PyObject *sigtools_convolve2d(PyObject *NPY_UNUSED(dummy), PyObject *args
     ret = pylab_convolve_2d (PyArray_DATA(ain1),      /* Input data Ns[0] x Ns[1] */
 		             PyArray_STRIDES(ain1),   /* Input strides */
 		             PyArray_DATA(aout),      /* Output data */
-		             PyArray_STRIDES(aout),   /* Ouput strides */
+		             PyArray_STRIDES(aout),   /* Output strides */
 		             PyArray_DATA(ain2),      /* coefficients in filter */
 		             PyArray_STRIDES(ain2),   /* coefficients strides */ 
 		             PyArray_DIMS(ain2),      /* Size of kernel Nwin[2] */
 			     PyArray_DIMS(ain1),      /* Size of image Ns[0] x Ns[1] */
 		             flag,                    /* convolution parameters */
-		             PyArray_DATA(newfill));  /* fill value */
+		             PyArray_DATA(afill));    /* fill value */
 
 
     switch (ret) {
@@ -1114,7 +1147,6 @@ static PyObject *sigtools_convolve2d(PyObject *NPY_UNUSED(dummy), PyObject *args
       Py_DECREF(ain1);
       Py_DECREF(ain2);
       Py_XDECREF(afill);
-      Py_XDECREF(newfill);
       return (PyObject *)aout;
       break;
     case -5:
@@ -1141,7 +1173,6 @@ fail:
     Py_XDECREF(ain2);
     Py_XDECREF(aout);
     Py_XDECREF(afill);
-    Py_XDECREF(newfill);
     return NULL;
 }
 
@@ -1160,111 +1191,114 @@ static PyObject *sigtools_order_filterND(PyObject *NPY_UNUSED(dummy),
 }
 
 
+static char doc_remez[] =
+    "h = _remez(numtaps, bands, des, weight, type, fs, maxiter, grid_density)\n"
+    "  returns the optimal (in the Chebyshev/minimax sense) FIR filter impulse\n"
+    "  response given a set of band edges, the desired response on those bands,\n"
+    "  and the weight given to the error in those bands.  Bands is a monotonic\n"
+    "  vector with band edges given in frequency domain where fs is the sampling\n"
+    "  frequency.";
 
-static char doc_remez[] = "h = _remez(numtaps, bands, des, weight, type, Hz, maxiter, grid_density) \n  returns the optimal (in the Chebyshev/minimax sense) FIR filter impulse \n  response given a set of band edges, the desired response on those bands,\n  and the weight given to the error in those bands.  Bands is a monotonic\n   vector with band edges given in frequency domain where Hz is the sampling\n   frequency.";
- 
-static PyObject *sigtools_remez(PyObject *NPY_UNUSED(dummy), PyObject *args) {
-        PyObject *bands, *des, *weight;
-        int k, numtaps, numbands, type = BANDPASS, err; 
-	PyArrayObject *a_bands=NULL, *a_des=NULL, *a_weight=NULL;
-	PyArrayObject *h=NULL;
-	intp ret_dimens; int maxiter = 25, grid_density = 16;
-	double oldvalue, *dptr, Hz = 1.0;
-        char mystr[255];
-       
+static PyObject *sigtools_remez(PyObject *NPY_UNUSED(dummy), PyObject *args)
+{
+    PyObject *bands, *des, *weight;
+    int k, numtaps, numbands, type = BANDPASS, err; 
+    PyArrayObject *a_bands=NULL, *a_des=NULL, *a_weight=NULL;
+    PyArrayObject *h=NULL;
+    intp ret_dimens; int maxiter = 25, grid_density = 16;
+    double oldvalue, *dptr, fs = 1.0;
+    char mystr[255];
 
+    if (!PyArg_ParseTuple(args, "iOOO|idii", &numtaps, &bands, &des, &weight, 
+                          &type, &fs, &maxiter, &grid_density)) {
+        return NULL;
+    }
 
-        if (!PyArg_ParseTuple(args, "iOOO|idii", &numtaps, &bands, &des, &weight, 
-                              &type, &Hz, &maxiter, &grid_density))
-	    return NULL;
-
-	if (type != BANDPASS && type != DIFFERENTIATOR && type != HILBERT) {
-	    PyErr_SetString(PyExc_ValueError,
-	                    "The type must be BANDPASS, DIFFERENTIATOR, or HILBERT.");
-	  return NULL;
-	}
-	
-	if (numtaps < 2) {
-	    PyErr_SetString(PyExc_ValueError,
-                            "The number of taps must be greater than 1.");
-	  return NULL;
-	}
-	
-	
-	a_bands = (PyArrayObject *)PyArray_ContiguousFromObject(bands, NPY_DOUBLE,1,1);
-	if (a_bands == NULL) goto fail;
-	a_des = (PyArrayObject *)PyArray_ContiguousFromObject(des, NPY_DOUBLE,1,1);
-	if (a_des == NULL) goto fail;
-	a_weight = (PyArrayObject *)PyArray_ContiguousFromObject(weight, NPY_DOUBLE,1,1);
-	if (a_weight == NULL) goto fail;
-
-
-	numbands = PyArray_DIMS(a_des)[0];
-	if ((PyArray_DIMS(a_bands)[0] != 2*numbands) || 
-            (PyArray_DIMS(a_weight)[0] != numbands)) {
-	  PyErr_SetString(PyExc_ValueError,
-			  "The inputs desired and weight must have same length.\n  " \
-                          "The input bands must have twice this length.");
-	  goto fail;
+    if (type != BANDPASS && type != DIFFERENTIATOR && type != HILBERT) {
+        PyErr_SetString(PyExc_ValueError,
+                        "The type must be BANDPASS, DIFFERENTIATOR, or HILBERT.");
+        return NULL;
 	}
 
-      /* 
-       * Check the bands input to see if it is monotonic, divide by 
-       * Hz to take from range 0 to 0.5 and check to see if in that range
-       */ 
-	dptr = (double *)PyArray_DATA(a_bands);
-	oldvalue = 0;
-	for (k=0; k < 2*numbands; k++) {
-	  if (*dptr < oldvalue) {
-	    PyErr_SetString(PyExc_ValueError,
-			  "Bands must be monotonic starting at zero.");
-	    goto fail;
-	  }
-	  if (*dptr * 2 > Hz) {
-	    PyErr_SetString(PyExc_ValueError,
-			  "Band edges should be less than 1/2 the sampling frequency");
-	    goto fail;
-	  }
-	  oldvalue = *dptr;
-	  *dptr = oldvalue / Hz;  /* Change so that sampling frequency is 1.0 */
-	  dptr++;
-	}
+    if (numtaps < 2) {
+        PyErr_SetString(PyExc_ValueError,
+                        "The number of taps must be greater than 1.");
+        return NULL;
+    }
 
-	ret_dimens = numtaps;
-	h = (PyArrayObject *)PyArray_SimpleNew(1, &ret_dimens, NPY_DOUBLE);
-	if (h == NULL) goto fail;
+    a_bands = (PyArrayObject *)PyArray_ContiguousFromObject(bands, NPY_DOUBLE,1,1);
+    if (a_bands == NULL) goto fail;
+    a_des = (PyArrayObject *)PyArray_ContiguousFromObject(des, NPY_DOUBLE,1,1);
+    if (a_des == NULL) goto fail;
+    a_weight = (PyArrayObject *)PyArray_ContiguousFromObject(weight, NPY_DOUBLE,1,1);
+    if (a_weight == NULL) goto fail;
 
-	err=pre_remez((double *)PyArray_DATA(h), numtaps, numbands, 
-                      (double *)PyArray_DATA(a_bands),
-                      (double *)PyArray_DATA(a_des),
-                      (double *)PyArray_DATA(a_weight),
-                      type, maxiter, grid_density);
-        if (err < 0) {
-	  if (err == -1) {
-            sprintf(mystr, "Failure to converge after %d iterations.\n", maxiter);
-	    PyErr_SetString(PyExc_ValueError, mystr);
-	    goto fail;
-	  }
-	  else if (err == -2) {
-	    PyErr_NoMemory();
+    numbands = PyArray_DIMS(a_des)[0];
+    if ((PyArray_DIMS(a_bands)[0] != 2*numbands) || 
+        (PyArray_DIMS(a_weight)[0] != numbands)) {
+	    PyErr_SetString(PyExc_ValueError,
+                        "The inputs desired and weight must have same length.\n  "
+                        "The input bands must have twice this length.");
+        goto fail;
+    }
+
+    /* 
+     * Check the bands input to see if it is monotonic, divide by 
+     * fs to take from range 0 to 0.5 and check to see if in that range
+     */ 
+    dptr = (double *)PyArray_DATA(a_bands);
+    oldvalue = 0;
+    for (k=0; k < 2*numbands; k++) {
+        if (*dptr < oldvalue) {
+            PyErr_SetString(PyExc_ValueError,
+                            "Bands must be monotonic starting at zero.");
             goto fail;
-	  }
-	}
+        }
+        if (*dptr * 2 > fs) {
+            PyErr_SetString(PyExc_ValueError,
+                            "Band edges should be less than 1/2 the sampling frequency");
+            goto fail;
+        }
+        oldvalue = *dptr;
+        *dptr = oldvalue / fs;  /* Change so that sampling frequency is 1.0 */
+        dptr++;
+    }
 
-	Py_DECREF(a_bands);
-	Py_DECREF(a_des);
-	Py_DECREF(a_weight);
+    ret_dimens = numtaps;
+    h = (PyArrayObject *)PyArray_SimpleNew(1, &ret_dimens, NPY_DOUBLE);
+    if (h == NULL) goto fail;
+
+    err = pre_remez((double *)PyArray_DATA(h), numtaps, numbands, 
+                    (double *)PyArray_DATA(a_bands),
+                    (double *)PyArray_DATA(a_des),
+                    (double *)PyArray_DATA(a_weight),
+                    type, maxiter, grid_density);
+    if (err < 0) {
+        if (err == -1) {
+            sprintf(mystr, "Failure to converge after %d iterations.\n", maxiter);
+	        PyErr_SetString(PyExc_ValueError, mystr);
+	        goto fail;
+        }
+        else if (err == -2) {
+            PyErr_NoMemory();
+            goto fail;
+        }
+    }
+
+    Py_DECREF(a_bands);
+    Py_DECREF(a_des);
+    Py_DECREF(a_weight);
 
 	return PyArray_Return(h);
 
- fail:
-	Py_XDECREF(a_bands);
-	Py_XDECREF(a_des);
-	Py_XDECREF(a_weight);
-	Py_XDECREF(h);
-	return NULL;
+fail:
+    Py_XDECREF(a_bands);
+    Py_XDECREF(a_des);
+    Py_XDECREF(a_weight);
+    Py_XDECREF(h);
+    return NULL;
 }
-   
+
 static char doc_median2d[] = "filt = _median2d(data, size)";
 
 extern void f_medfilt2(float*,float*,intp*,intp*);
