@@ -3,16 +3,21 @@
 
 from __future__ import division, print_function, absolute_import
 
+import warnings
 import numpy as np
+from numpy.linalg import LinAlgError
 from scipy._lib.six import xrange
-from scipy.linalg import get_blas_funcs, get_lapack_funcs, qr_insert, lstsq
+from scipy.linalg import get_blas_funcs, get_lapack_funcs
 from .utils import make_system
 
+from ._gcrotmk import _fgmres
+
 __all__ = ['lgmres']
 
 
 def lgmres(A, b, x0=None, tol=1e-5, maxiter=1000, M=None, callback=None,
-           inner_m=30, outer_k=3, outer_v=None, store_outer_Av=True):
+           inner_m=30, outer_k=3, outer_v=None, store_outer_Av=True,
+           prepend_outer_v=False, atol=None):
     """
     Solve a matrix equation using the LGMRES algorithm.
 
@@ -28,9 +33,14 @@ def lgmres(A, b, x0=None, tol=1e-5, maxiter=1000, M=None, callback=None,
         Right hand side of the linear system. Has shape (N,) or (N,1).
     x0  : {array, matrix}
         Starting guess for the solution.
-    tol : float, optional
-        Tolerance to achieve. The algorithm terminates when either the relative
-        or the absolute residual is below `tol`.
+    tol, atol : float, optional
+        Tolerances for convergence, ``norm(residual) <= max(tol*norm(b), atol)``.
+        The default for ``atol`` is `tol`.
+
+        .. warning::
+
+           The default value for `atol` will be changed in a future release.
+           For future compatibility, specify `atol` explicitly.
     maxiter : int, optional
         Maximum number of iterations.  Iteration will stop after maxiter
         steps even if the specified tolerance has not been achieved.
@@ -61,6 +71,9 @@ def lgmres(A, b, x0=None, tol=1e-5, maxiter=1000, M=None, callback=None,
     store_outer_Av : bool, optional
         Whether LGMRES should store also A*v in addition to vectors `v`
         in the `outer_v` list. Default is True.
+    prepend_outer_v : bool, optional 
+        Whether to put outer_v augmentation vectors before Krylov iterates.
+        In standard LGMRES, prepend_outer_v=False.
 
     Returns
     -------
@@ -91,17 +104,36 @@ def lgmres(A, b, x0=None, tol=1e-5, maxiter=1000, M=None, callback=None,
 
     References
     ----------
-    .. [1] A.H. Baker and E.R. Jessup and T. Manteuffel,
-             SIAM J. Matrix Anal. Appl. 26, 962 (2005).
-    .. [2] A.H. Baker, PhD thesis, University of Colorado (2003).
-             http://amath.colorado.edu/activities/thesis/allisonb/Thesis.ps
+    .. [1] A.H. Baker and E.R. Jessup and T. Manteuffel, "A Technique for
+             Accelerating the Convergence of Restarted GMRES", SIAM J. Matrix
+             Anal. Appl. 26, 962 (2005).
+    .. [2] A.H. Baker, "On Improving the Performance of the Linear Solver
+             restarted GMRES", PhD thesis, University of Colorado (2003).
 
+    Examples
+    --------
+    >>> from scipy.sparse import csc_matrix
+    >>> from scipy.sparse.linalg import lgmres
+    >>> A = csc_matrix([[3, 2, 0], [1, -1, 0], [0, 5, 1]], dtype=float)
+    >>> b = np.array([2, 4, -1], dtype=float)
+    >>> x, exitCode = lgmres(A, b)
+    >>> print(exitCode)            # 0 indicates successful convergence
+    0
+    >>> np.allclose(A.dot(x), b)
+    True
     """
     A,M,x,b,postprocess = make_system(A,M,x0,b)
 
     if not np.isfinite(b).all():
         raise ValueError("RHS must contain only finite numbers")
 
+    if atol is None:
+        warnings.warn("scipy.sparse.linalg.lgmres called without specifying `atol`. "
+                      "The default value will change in the future. To preserve "
+                      "current behavior, set ``atol=tol``.",
+                      category=DeprecationWarning, stacklevel=2)
+        atol = tol
+
     matvec = A.matvec
     psolve = M.matvec
 
@@ -112,8 +144,7 @@ def lgmres(A, b, x0=None, tol=1e-5, maxiter=1000, M=None, callback=None,
     nrm2 = get_blas_funcs('nrm2', [b])
 
     b_norm = nrm2(b)
-    if b_norm == 0:
-        b_norm = 1
+    ptol_max_factor = 1.0
 
     for k_outer in xrange(maxiter):
         r_outer = matvec(x) - b
@@ -128,153 +159,52 @@ def lgmres(A, b, x0=None, tol=1e-5, maxiter=1000, M=None, callback=None,
                 x = x.astype(r_outer.dtype)
             axpy, dot, scal, nrm2 = get_blas_funcs(['axpy', 'dot', 'scal', 'nrm2'],
                                                    (x, r_outer))
-            trtrs = get_lapack_funcs('trtrs', (x, r_outer))
 
         # -- check stopping condition
         r_norm = nrm2(r_outer)
-        if r_norm <= tol * b_norm or r_norm <= tol:
+        if r_norm <= max(atol, tol * b_norm):
             break
 
         # -- inner LGMRES iteration
-        vs0 = -psolve(r_outer)
-        inner_res_0 = nrm2(vs0)
+        v0 = -psolve(r_outer)
+        inner_res_0 = nrm2(v0)
 
         if inner_res_0 == 0:
             rnorm = nrm2(r_outer)
             raise RuntimeError("Preconditioner returned a zero vector; "
                                "|v| ~ %.1g, |M v| = 0" % rnorm)
 
-        vs0 = scal(1.0/inner_res_0, vs0)
-        vs = [vs0]
-        ws = []
-        y = None
+        v0 = scal(1.0/inner_res_0, v0)
 
-        # H is stored in QR factorized form
-        Q = np.ones((1, 1), dtype=vs0.dtype)
-        R = np.zeros((1, 0), dtype=vs0.dtype)
+        ptol = min(ptol_max_factor, max(atol, tol*b_norm)/r_norm)
 
-        eps = np.finfo(vs0.dtype).eps
-
-        breakdown = False
-
-        for j in xrange(1, 1 + inner_m + len(outer_v)):
-            # -- Arnoldi process:
-            #
-            #    Build an orthonormal basis V and matrices W and H such that
-            #        A W = V H
-            #    Columns of W, V, and H are stored in `ws`, `vs` and `hs`.
-            #
-            #    The first column of V is always the residual vector, `vs0`;
-            #    V has *one more column* than the other of the three matrices.
-            #
-            #    The other columns in V are built by feeding in, one
-            #    by one, some vectors `z` and orthonormalizing them
-            #    against the basis so far. The trick here is to
-            #    feed in first some augmentation vectors, before
-            #    starting to construct the Krylov basis on `v0`.
-            #
-            #    It was shown in [BJM]_ that a good choice (the LGMRES choice)
-            #    for these augmentation vectors are the `dx` vectors obtained
-            #    from a couple of the previous restart cycles.
-            #
-            #    Note especially that while `vs0` is always the first
-            #    column in V, there is no reason why it should also be
-            #    the first column in W. (In fact, below `vs0` comes in
-            #    W only after the augmentation vectors.)
-            #
-            #    The rest of the algorithm then goes as in GMRES, one
-            #    solves a minimization problem in the smaller subspace
-            #    spanned by W (range) and V (image).
-            #
-
-            #     ++ evaluate
-            v_new = None
-            if j < len(outer_v) + 1:
-                z, v_new = outer_v[j-1]
-            elif j == len(outer_v) + 1:
-                z = vs0
-            else:
-                z = vs[-1]
-
-            if v_new is None:
-                v_new = psolve(matvec(z))
-            else:
-                # Note: v_new is modified in-place below. Must make a
-                # copy to ensure that the outer_v vectors are not
-                # clobbered.
-                v_new = v_new.copy()
-
-            #     ++ orthogonalize
-            v_new_norm = nrm2(v_new)
-
-            hcur = np.zeros(j+1, dtype=Q.dtype)
-            for i, v in enumerate(vs):
-                alpha = dot(v, v_new)
-                hcur[i] = alpha
-                v_new = axpy(v, v_new, v.shape[0], -alpha)  # v_new -= alpha*v
-            hcur[-1] = nrm2(v_new)
-
-            with np.errstate(over='ignore', divide='ignore'):
-                # Careful with denormals
-                alpha = 1/hcur[-1]
-
-            if np.isfinite(alpha):
-                v_new = scal(alpha, v_new)
-
-            if not (hcur[-1] > eps * v_new_norm):
-                # v_new essentially in the span of previous vectors,
-                # or we have nans. Bail out after updating the QR
-                # solution.
-                breakdown = True
-
-            vs.append(v_new)
-            ws.append(z)
-
-            # -- GMRES optimization problem
-
-            # Add new column to H=Q*R, padding other columns with zeros
-
-            Q2 = np.zeros((j+1, j+1), dtype=Q.dtype, order='F')
-            Q2[:j,:j] = Q
-            Q2[j,j] = 1
-
-            R2 = np.zeros((j+1, j-1), dtype=R.dtype, order='F')
-            R2[:j,:] = R
-
-            Q, R = qr_insert(Q2, R2, hcur, j-1, which='col',
-                             overwrite_qru=True, check_finite=False)
-
-            # Transformed least squares problem
-            # || Q R y - inner_res_0 * e_1 ||_2 = min!
-            # Since R = [R'; 0], solution is y = inner_res_0 (R')^{-1} (Q^H)[:j,0]
-
-            # Residual is immediately known
-            inner_res = abs(Q[0,-1]) * inner_res_0
-
-            # -- check for termination
-            if inner_res <= tol * inner_res_0 or breakdown:
-                break
-
-        if not np.isfinite(R[j-1,j-1]):
-            # nans encountered, bail out
-            return postprocess(x), k_outer + 1
-
-        # -- Get the LSQ problem solution
-        #
-        # The problem is triangular, but the condition number may be
-        # bad (or in case of breakdown the last diagonal entry may be
-        # zero), so use lstsq instead of trtrs.
-        y, _, _, _, = lstsq(R[:j,:j], Q[0,:j].conj())
-        y *= inner_res_0
-
-        if not np.isfinite(y).all():
+        try:
+            Q, R, B, vs, zs, y, pres = _fgmres(matvec,
+                                               v0,
+                                               inner_m,
+                                               lpsolve=psolve,
+                                               atol=ptol,
+                                               outer_v=outer_v,
+                                               prepend_outer_v=prepend_outer_v)
+            y *= inner_res_0
+            if not np.isfinite(y).all():
+                # Overflow etc. in computation. There's no way to
+                # recover from this, so we have to bail out.
+                raise LinAlgError()
+        except LinAlgError:
             # Floating point over/underflow, non-finite result from
             # matmul etc. -- report failure.
             return postprocess(x), k_outer + 1
 
+        # Inner loop tolerance control
+        if pres > ptol:
+            ptol_max_factor = min(1.0, 1.5 * ptol_max_factor)
+        else:
+            ptol_max_factor = max(1e-16, 0.25 * ptol_max_factor)
+
         # -- GMRES terminated: eval solution
-        dx = ws[0]*y[0]
-        for w, yc in zip(ws[1:], y[1:]):
+        dx = zs[0]*y[0]
+        for w, yc in zip(zs[1:], y[1:]):
             dx = axpy(w, dx, dx.shape[0], yc)  # dx += w*yc
 
         # -- Store LGMRES augmentation vectors
