@@ -8,68 +8,64 @@
 #include "csr.h"
 #include "dense.h"
 
+static inline npy_intp diagonal_size(const npy_intp k,
+                                     const npy_intp rows,
+                                     const npy_intp cols)
+{
+    return std::min(rows + std::min(k, (npy_intp)0),
+                    cols - std::max(k, (npy_intp)0));
+}
+
+
 template <class I, class T>
-void bsr_diagonal(const I n_brow,
-                  const I n_bcol, 
+void bsr_diagonal(const I k,
+                  const I n_brow,
+                  const I n_bcol,
                   const I R,
                   const I C,
-	              const I Ap[], 
-	              const I Aj[], 
-	              const T Ax[],
-	                    T Yx[])
+                  const I Ap[],
+                  const I Aj[],
+                  const T Ax[],
+                        T Yx[])
 {
-    const npy_intp D  = std::min((npy_intp)R*n_brow,
-                                 (npy_intp)C*n_bcol);
-    const npy_intp RC = (npy_intp)R*C;
+    const npy_intp RC = R * C;
+    const npy_intp D = diagonal_size(k, (npy_intp)n_brow * R,
+                                        (npy_intp)n_bcol * C);
+    const npy_intp first_row = (k >= 0) ? 0 : -k;
+    /* First and next-to-last brows of the diagonal. */
+    const npy_intp first_brow = first_row / R;
+    const npy_intp last_brow = (first_row + D - 1) / R + 1;
 
-    for (npy_intp i = 0; i < D; i++){
-        Yx[i] = 0;
-    }
+    for (npy_intp brow = first_brow; brow < last_brow; ++brow) {
+        /* First and next-to-last bcols of the diagonal in this brow. */
+        const npy_intp first_bcol = (brow * R + k) / C;
+        const npy_intp last_bcol = ((brow + 1) * R + k - 1) / C + 1;
 
-    if ( R == C ){
-        //main diagonal with square blocks
-        const I end = std::min(n_brow,n_bcol);
-        for(I i = 0; i < end; i++){
-            for(I jj = Ap[i]; jj < Ap[i+1]; jj++){
-                if (i == Aj[jj]){
-                    npy_intp row = (npy_intp)R*i;
-                    const T * val = Ax + (npy_intp)RC*jj;
-                    for(I bi = 0; bi < R; bi++){
-                        Yx[row + bi] = *val;
-                        val += C + 1;
-                    }
+        for (npy_intp jj = Ap[brow]; jj < Ap[brow + 1]; ++jj) {
+            const npy_intp bcol = Aj[jj];
+
+            if (first_bcol <= bcol && bcol < last_bcol) {
+                /*
+                 * Compute and extract diagonal of block corresponding to the
+                 * k-th overall diagonal and add it to output in right place.
+                 */
+                const npy_intp block_k = brow * R + k - bcol * C;
+                const npy_intp block_D = diagonal_size(block_k, R, C);
+                const npy_intp block_first_row = (block_k >= 0) ? 0 : -block_k;
+                const npy_intp Y_idx = brow * R + block_first_row - first_row;
+                const npy_intp Ax_idx = RC * jj +
+                                        ((block_k >= 0) ? block_k :
+                                                          -C * block_k);
+
+                for (npy_intp kk = 0; kk < block_D; ++kk) {
+                    Yx[Y_idx + kk] += Ax[Ax_idx + kk * (C + 1)];
                 }
-            }
-        }
-    } 
-    else 
-    {
-        //This could be made faster
-        const I end = (D/R) + (D % R == 0 ? 0 : 1);
-        for(I i = 0; i < end; i++){
-            for(I jj = Ap[i]; jj < Ap[i+1]; jj++){
-                const npy_intp base_row = (npy_intp)R*i;
-                const npy_intp base_col = (npy_intp)C*Aj[jj];
-                const T * base_val = Ax + (npy_intp)RC*jj;
 
-                for(I bi = 0; bi < R; bi++){
-                    const I row = base_row + bi;
-                    if (row >= D) break;
-
-                    for(I bj = 0; bj < C; bj++){
-                        const I col = base_col + bj;
-                        if (row == col){
-                            Yx[row] = base_val[(npy_intp)bi*C + bj];
-                        }
-                    }
-                }
             }
         }
     }
 }
 
-
-
 /*
  * Scale the rows of a BSR matrix *in place*
  *
@@ -78,13 +74,13 @@ void bsr_diagonal(const I n_brow,
  */
 template <class I, class T>
 void bsr_scale_rows(const I n_brow,
-                    const I n_bcol, 
+                    const I n_bcol,
                     const I R,
                     const I C,
-	                const I Ap[], 
-	                const I Aj[], 
-	                      T Ax[],
-	                const T Xx[])
+                    const I Ap[],
+                    const I Aj[],
+                          T Ax[],
+                    const T Xx[])
 {
     const npy_intp RC = (npy_intp)R*C;
 
@@ -109,13 +105,13 @@ void bsr_scale_rows(const I n_brow,
  */
 template <class I, class T>
 void bsr_scale_columns(const I n_brow,
-                       const I n_bcol, 
+                       const I n_bcol,
                        const I R,
                        const I C,
-	                   const I Ap[], 
-	                   const I Aj[], 
-	                         T Ax[],
-	                   const T Xx[])
+                       const I Ap[],
+                       const I Aj[],
+                             T Ax[],
+                       const T Xx[])
 {
     const I bnnz = Ap[n_brow];
     const npy_intp RC  = (npy_intp)R*C;
@@ -149,19 +145,19 @@ void bsr_scale_columns(const I n_brow,
  */
 template <class I, class T>
 void bsr_sort_indices(const I n_brow,
-	                  const I n_bcol, 
+                      const I n_bcol,
                       const I R,
                       const I C,
-	                        I Ap[], 
-	                        I Aj[], 
-	                        T Ax[])
-{  
+                            I Ap[],
+                            I Aj[],
+                            T Ax[])
+{
     if( R == 1 && C == 1 ){
         csr_sort_indices(n_brow, Ap, Aj, Ax);
         return;
     }
-    
-    
+
+
     const I nblks = Ap[n_brow];
     const npy_intp RC    = (npy_intp)R*C;
     const npy_intp nnz   = (npy_intp)RC*nblks;
@@ -205,25 +201,25 @@ void bsr_sort_indices(const I n_brow,
  * Note:
  *   Output arrays Bp, Bj, Bx must be preallocated
  *
- * Note: 
+ * Note:
  *   Input:  column indices *are not* assumed to be in sorted order
  *   Output: row indices *will be* in sorted order
  *
  *   Complexity: Linear.  Specifically O(nnz(A) + max(n_row,n_col))
- * 
+ *
  */
 template <class I, class T>
 void bsr_transpose(const I n_brow,
-	               const I n_bcol, 
+                   const I n_bcol,
                    const I R,
                    const I C,
-	               const I Ap[], 
-	               const I Aj[], 
-	               const T Ax[],
-	                     I Bp[],
-	                     I Bj[],
-	                     T Bx[])
-{  
+                   const I Ap[],
+                   const I Aj[],
+                   const T Ax[],
+                         I Bp[],
+                         I Bj[],
+                         T Bx[])
+{
     const I nblks = Ap[n_brow];
     const npy_intp RC    = (npy_intp)R*C;
 
@@ -250,11 +246,11 @@ void bsr_transpose(const I n_brow,
 
 
 template <class I, class T>
-void bsr_matmat_pass2(const I n_brow,  const I n_bcol, 
+void bsr_matmat_pass2(const I n_brow,  const I n_bcol,
                       const I R,       const I C,       const I N,
-      	              const I Ap[],    const I Aj[],    const T Ax[],
-      	              const I Bp[],    const I Bj[],    const T Bx[],
-      	                    I Cp[],          I Cj[],          T Cx[])
+                      const I Ap[],    const I Aj[],    const T Ax[],
+                      const I Bp[],    const I Bj[],    const T Bx[],
+                            I Cp[],          I Cj[],          T Cx[])
 {
     assert(R > 0 && C > 0 && N > 0);
 
@@ -269,7 +265,7 @@ void bsr_matmat_pass2(const I n_brow,  const I n_bcol,
     const npy_intp NC = (npy_intp)N*C;
 
     std::fill( Cx, Cx + RC * Cp[n_brow], 0 ); //clear output array
- 
+
     std::vector<I>  next(n_bcol,-1);
     std::vector<T*> mats(n_bcol);
 
@@ -291,7 +287,7 @@ void bsr_matmat_pass2(const I n_brow,  const I n_bcol,
                 I k = Bj[kk];
 
                 if(next[k] == -1){
-                    next[k] = head;                        
+                    next[k] = head;
                     head = k;
                     Cj[nnz] = k;
                     mats[k] = Cx + RC*nnz;
@@ -304,10 +300,10 @@ void bsr_matmat_pass2(const I n_brow,  const I n_bcol,
 
                 gemm(R, C, N, A, B, mats[k]);
             }
-        }         
+        }
 
         for(I jj = 0; jj < length; jj++){
-            I temp = head;                
+            I temp = head;
             head = next[head];
             next[temp] = -1; //clear arrays
         }
@@ -337,14 +333,14 @@ bool is_nonzero_block(const T block[], const I blocksize){
  * unsorted column indices within a given row.
  *
  * Refer to bsr_binop_bsr() for additional information
- *   
+ *
  * Note:
  *   Output arrays Cp, Cj, and Cx must be preallocated
  *   If nnz(C) is not known a priori, a conservative bound is:
  *          nnz(C) <= nnz(A) + nnz(B)
  *
- * Note: 
- *   Input:  A and B column indices are not assumed to be in sorted order 
+ * Note:
+ *   Input:  A and B column indices are not assumed to be in sorted order
  *   Output: C column indices are not generally in sorted order
  *           C will not contain any duplicate entries or explicit zeros.
  *
@@ -352,9 +348,9 @@ bool is_nonzero_block(const T block[], const I blocksize){
 template <class I, class T, class T2, class bin_op>
 void bsr_binop_bsr_general(const I n_brow, const I n_bcol,
                            const I R,      const I C,
-                           const I Ap[],  const I Aj[],  const T Ax[],
-                           const I Bp[],  const I Bj[],  const T Bx[],
-                                 I Cp[],        I Cj[],       T2 Cx[],
+                           const I Ap[],   const I Aj[],   const T Ax[],
+                           const I Bp[],   const I Bj[],   const T Bx[],
+                                 I Cp[],         I Cj[],        T2 Cx[],
                            const bin_op& op)
 {
     //Method that works for duplicate and/or unsorted indices
@@ -379,7 +375,7 @@ void bsr_binop_bsr_general(const I n_brow, const I n_bcol,
                 A_row[RC*j + n] += Ax[RC*jj + n];
 
             if(next[j] == -1){
-                next[j] = head;                       
+                next[j] = head;
                 head = j;
                 length++;
             }
@@ -393,7 +389,7 @@ void bsr_binop_bsr_general(const I n_brow, const I n_bcol,
                 B_row[RC*j + n] += Bx[RC*jj + n];
 
             if(next[j] == -1){
-                next[j] = head;                       
+                next[j] = head;
                 head = j;
                 length++;
             }
@@ -415,36 +411,36 @@ void bsr_binop_bsr_general(const I n_brow, const I n_bcol,
                 B_row[RC*head + n] = 0;
             }
 
-            I temp = head;               
+            I temp = head;
             head = next[head];
             next[temp] = -1;
         }
-        
+
         Cp[i + 1] = nnz;
     }
 }
 
 
 /*
- * Compute C = A (binary_op) B for BSR matrices that are in the 
+ * Compute C = A (binary_op) B for BSR matrices that are in the
  * canonical BSR format.  Specifically, this method requires that
  * the rows of the input matrices are free of duplicate column indices
  * and that the column indices are in sorted order.
  *
  * Refer to bsr_binop_bsr() for additional information
  *
- * Note: 
- *   Input:  A and B column indices are assumed to be in sorted order 
+ * Note:
+ *   Input:  A and B column indices are assumed to be in sorted order
  *   Output: C column indices will be in sorted order
  *           Cx will not contain any zero entries
  *
  */
 template <class I, class T, class T2, class bin_op>
-void bsr_binop_bsr_canonical(const I n_brow, const I n_bcol, 
-                             const I R,      const I C, 
-                             const I Ap[],  const I Aj[],  const T Ax[],
-                             const I Bp[],  const I Bj[],  const T Bx[],
-                                   I Cp[],        I Cj[],       T2 Cx[],
+void bsr_binop_bsr_canonical(const I n_brow, const I n_bcol,
+                             const I R,      const I C,
+                             const I Ap[],   const I Aj[],   const T Ax[],
+                             const I Bp[],   const I Bj[],   const T Bx[],
+                                   I Cp[],         I Cj[],        T2 Cx[],
                              const bin_op& op)
 {
     const npy_intp RC = (npy_intp)R*C;
@@ -475,7 +471,7 @@ void bsr_binop_bsr_canonical(const I n_brow, const I n_bcol,
                     nnz++;
                 }
 
-                A_pos++; 
+                A_pos++;
                 B_pos++;
             } else if (A_j < B_j) {
                 for(I n = 0; n < RC; n++){
@@ -488,7 +484,7 @@ void bsr_binop_bsr_canonical(const I n_brow, const I n_bcol,
                     nnz++;
                 }
 
-                A_pos++; 
+                A_pos++;
             } else {
                 //B_j < A_j
                 for(I n = 0; n < RC; n++){
@@ -516,7 +512,7 @@ void bsr_binop_bsr_canonical(const I n_brow, const I n_bcol,
                 nnz++;
             }
 
-            A_pos++; 
+            A_pos++;
         }
         while(B_pos < B_end){
             for(I n = 0; n < RC; n++){
@@ -538,7 +534,7 @@ void bsr_binop_bsr_canonical(const I n_brow, const I n_bcol,
 
 
 /*
- * Compute C = A (binary_op) B for CSR matrices A,B where the column 
+ * Compute C = A (binary_op) B for CSR matrices A,B where the column
  * indices with the rows of A and B are known to be sorted.
  *
  *   binary_op(x,y) - binary operator to apply elementwise
@@ -556,28 +552,28 @@ void bsr_binop_bsr_canonical(const I n_brow, const I n_bcol,
  *   I    Cp[n_row+1] - row pointer
  *   I    Cj[nnz(C)]  - column indices
  *   T    Cx[nnz(C)]  - nonzeros
- *   
+ *
  * Note:
  *   Output arrays Cp, Cj, and Cx must be preallocated
  *   If nnz(C) is not known a priori, a conservative bound is:
  *          nnz(C) <= nnz(A) + nnz(B)
  *
- * Note: 
+ * Note:
  *   Input:  A and B column indices are not assumed to be in sorted order.
  *   Output: C column indices will be in sorted if both A and B have sorted indices.
  *           Cx will not contain any zero entries
  *
  */
 template <class I, class T, class T2, class bin_op>
-void bsr_binop_bsr(const I n_brow, const I n_bcol, 
-                   const I R,     const I C, 
-                   const I Ap[],  const I Aj[],  const T Ax[],
-                   const I Bp[],  const I Bj[],  const T Bx[],
-                         I Cp[],        I Cj[],       T2 Cx[],
+void bsr_binop_bsr(const I n_brow, const I n_bcol,
+                   const I R,      const I C,
+                   const I Ap[],   const I Aj[],   const T Ax[],
+                   const I Bp[],   const I Bj[],   const T Bx[],
+                         I Cp[],         I Cj[],        T2 Cx[],
                    const bin_op& op)
 {
     assert( R > 0 && C > 0);
-    
+
     if( R == 1 && C == 1 ){
         //use CSR for 1x1 blocksize
         csr_binop_csr(n_brow, n_bcol, Ap, Aj, Ax, Bp, Bj, Bx, Cp, Cj, Cx, op);
@@ -594,52 +590,52 @@ void bsr_binop_bsr(const I n_brow, const I n_bcol,
 
 /* element-wise binary operations */
 template <class I, class T, class T2>
-void bsr_ne_bsr(const I n_row, const I n_col, const I R, const I C, 
-                   const I Ap[], const I Aj[], const T Ax[],
-                   const I Bp[], const I Bj[], const T Bx[],
-                         I Cp[],       I Cj[],      T2 Cx[])
+void bsr_ne_bsr(const I n_row, const I n_col, const I R, const I C,
+                const I Ap[], const I Aj[], const T Ax[],
+                const I Bp[], const I Bj[], const T Bx[],
+                      I Cp[],       I Cj[],      T2 Cx[])
 {
     bsr_binop_bsr(n_row,n_col,R,C,Ap,Aj,Ax,Bp,Bj,Bx,Cp,Cj,Cx,std::not_equal_to<T>());
 }
 
 template <class I, class T, class T2>
-void bsr_lt_bsr(const I n_row, const I n_col, const I R, const I C, 
-                   const I Ap[], const I Aj[], const T Ax[],
-                   const I Bp[], const I Bj[], const T Bx[],
-                         I Cp[],       I Cj[],      T2 Cx[])
+void bsr_lt_bsr(const I n_row, const I n_col, const I R, const I C,
+                const I Ap[], const I Aj[], const T Ax[],
+                const I Bp[], const I Bj[], const T Bx[],
+                      I Cp[],       I Cj[],      T2 Cx[])
 {
     bsr_binop_bsr(n_row,n_col,R,C,Ap,Aj,Ax,Bp,Bj,Bx,Cp,Cj,Cx,std::less<T>());
 }
 
 template <class I, class T, class T2>
-void bsr_gt_bsr(const I n_row, const I n_col, const I R, const I C, 
-                   const I Ap[], const I Aj[], const T Ax[],
-                   const I Bp[], const I Bj[], const T Bx[],
-                         I Cp[],       I Cj[],      T2 Cx[])
+void bsr_gt_bsr(const I n_row, const I n_col, const I R, const I C,
+                const I Ap[], const I Aj[], const T Ax[],
+                const I Bp[], const I Bj[], const T Bx[],
+                      I Cp[],       I Cj[],      T2 Cx[])
 {
     bsr_binop_bsr(n_row,n_col,R,C,Ap,Aj,Ax,Bp,Bj,Bx,Cp,Cj,Cx,std::greater<T>());
 }
 
 template <class I, class T, class T2>
-void bsr_le_bsr(const I n_row, const I n_col, const I R, const I C, 
-                   const I Ap[], const I Aj[], const T Ax[],
-                   const I Bp[], const I Bj[], const T Bx[],
-                         I Cp[],       I Cj[],      T2 Cx[])
+void bsr_le_bsr(const I n_row, const I n_col, const I R, const I C,
+                const I Ap[], const I Aj[], const T Ax[],
+                const I Bp[], const I Bj[], const T Bx[],
+                      I Cp[],       I Cj[],      T2 Cx[])
 {
     bsr_binop_bsr(n_row,n_col,R,C,Ap,Aj,Ax,Bp,Bj,Bx,Cp,Cj,Cx,std::less_equal<T>());
 }
 
 template <class I, class T, class T2>
-void bsr_ge_bsr(const I n_row, const I n_col, const I R, const I C, 
-                   const I Ap[], const I Aj[], const T Ax[],
-                   const I Bp[], const I Bj[], const T Bx[],
-                         I Cp[],       I Cj[],      T2 Cx[])
+void bsr_ge_bsr(const I n_row, const I n_col, const I R, const I C,
+                const I Ap[], const I Aj[], const T Ax[],
+                const I Bp[], const I Bj[], const T Bx[],
+                      I Cp[],       I Cj[],      T2 Cx[])
 {
     bsr_binop_bsr(n_row,n_col,R,C,Ap,Aj,Ax,Bp,Bj,Bx,Cp,Cj,Cx,std::greater_equal<T>());
 }
 
 template <class I, class T>
-void bsr_elmul_bsr(const I n_row, const I n_col, const I R, const I C, 
+void bsr_elmul_bsr(const I n_row, const I n_col, const I R, const I C,
                    const I Ap[], const I Aj[], const T Ax[],
                    const I Bp[], const I Bj[], const T Bx[],
                          I Cp[],       I Cj[],       T Cx[])
@@ -658,7 +654,7 @@ void bsr_eldiv_bsr(const I n_row, const I n_col, const I R, const I C,
 
 
 template <class I, class T>
-void bsr_plus_bsr(const I n_row, const I n_col, const I R, const I C, 
+void bsr_plus_bsr(const I n_row, const I n_col, const I R, const I C,
                   const I Ap[], const I Aj[], const T Ax[],
                   const I Bp[], const I Bj[], const T Bx[],
                         I Cp[],       I Cj[],       T Cx[])
@@ -667,7 +663,7 @@ void bsr_plus_bsr(const I n_row, const I n_col, const I R, const I C,
 }
 
 template <class I, class T>
-void bsr_minus_bsr(const I n_row, const I n_col, const I R, const I C, 
+void bsr_minus_bsr(const I n_row, const I n_col, const I R, const I C,
                    const I Ap[], const I Aj[], const T Ax[],
                    const I Bp[], const I Bj[], const T Bx[],
                          I Cp[],       I Cj[],       T Cx[])
@@ -677,19 +673,19 @@ void bsr_minus_bsr(const I n_row, const I n_col, const I R, const I C,
 
 
 template <class I, class T>
-void bsr_maximum_bsr(const I n_row, const I n_col, const I R, const I C, 
-                  const I Ap[], const I Aj[], const T Ax[],
-                  const I Bp[], const I Bj[], const T Bx[],
-                        I Cp[],       I Cj[],       T Cx[])
+void bsr_maximum_bsr(const I n_row, const I n_col, const I R, const I C,
+                     const I Ap[], const I Aj[], const T Ax[],
+                     const I Bp[], const I Bj[], const T Bx[],
+                           I Cp[],       I Cj[],       T Cx[])
 {
     bsr_binop_bsr(n_row,n_col,R,C,Ap,Aj,Ax,Bp,Bj,Bx,Cp,Cj,Cx,maximum<T>());
 }
 
 template <class I, class T>
-void bsr_minimum_bsr(const I n_row, const I n_col, const I R, const I C, 
-                   const I Ap[], const I Aj[], const T Ax[],
-                   const I Bp[], const I Bj[], const T Bx[],
-                         I Cp[],       I Cj[],       T Cx[])
+void bsr_minimum_bsr(const I n_row, const I n_col, const I R, const I C,
+                     const I Ap[], const I Aj[], const T Ax[],
+                     const I Bp[], const I Bj[], const T Bx[],
+                           I Cp[],       I Cj[],       T Cx[])
 {
     bsr_binop_bsr(n_row,n_col,R,C,Ap,Aj,Ax,Bp,Bj,Bx,Cp,Cj,Cx,minimum<T>());
 }
@@ -700,15 +696,15 @@ void bsr_minimum_bsr(const I n_row, const I n_col, const I R, const I C,
 
 //template <class I, class T>
 //void bsr_tocsr(const I n_brow,
-//	           const I n_bcol, 
-//	           const I R, 
-//	           const I C, 
-//	           const I Ap[], 
-//	           const I Aj[], 
-//	           const T Ax[],
-//	                 I Bp[],
+//             const I n_bcol,
+//             const I R,
+//             const I C,
+//             const I Ap[],
+//             const I Aj[],
+//             const T Ax[],
+//                   I Bp[],
 //                     I Bj[]
-//	                 T Bx[])
+//                   T Bx[])
 //{
 //    const I RC = R*C;
 //
@@ -723,19 +719,19 @@ void bsr_minimum_bsr(const I n_row, const I n_col, const I R, const I C,
 
 template <class I, class T>
 void bsr_matvec(const I n_brow,
-	            const I n_bcol, 
-	            const I R, 
-	            const I C, 
-	            const I Ap[], 
-	            const I Aj[], 
-	            const T Ax[],
-	            const T Xx[],
-	                  T Yx[])
+                const I n_bcol,
+                const I R,
+                const I C,
+                const I Ap[],
+                const I Aj[],
+                const T Ax[],
+                const T Xx[],
+                      T Yx[])
 {
     assert(R > 0 && C > 0);
 
     if( R == 1 && C == 1 ){
-        //use CSR for 1x1 blocksize 
+        //use CSR for 1x1 blocksize
         csr_matvec(n_brow, n_bcol, Ap, Aj, Ax, Xx, Yx);
         return;
     }
@@ -774,20 +770,20 @@ void bsr_matvec(const I n_brow,
  */
 template <class I, class T>
 void bsr_matvecs(const I n_brow,
-	             const I n_bcol, 
+                 const I n_bcol,
                  const I n_vecs,
-	             const I R, 
-	             const I C, 
-	             const I Ap[], 
-	             const I Aj[], 
-	             const T Ax[],
-	             const T Xx[],
-	                   T Yx[])
+                 const I R,
+                 const I C,
+                 const I Ap[],
+                 const I Aj[],
+                 const T Ax[],
+                 const T Xx[],
+                       T Yx[])
 {
     assert(R > 0 && C > 0);
 
     if( R == 1 && C == 1 ){
-        //use CSR for 1x1 blocksize 
+        //use CSR for 1x1 blocksize
         csr_matvecs(n_brow, n_bcol, n_vecs, Ap, Aj, Ax, Xx, Yx);
         return;
     }
