@@ -12,27 +12,27 @@
  *   I  n_col         - number of columns in A
  *   I  Ap[n_row+1]   - column pointer
  *   I  Ai[nnz(A)]    - row indices
- *   T  Ax[n_col]     - nonzeros 
+ *   T  Ax[n_col]     - nonzeros
  *   T  Xx[n_col]     - input vector
  *
  * Output Arguments:
- *   T  Yx[n_row]     - output vector 
+ *   T  Yx[n_row]     - output vector
  *
  * Note:
  *   Output array Yx must be preallocated
- *   
+ *
  *   Complexity: Linear.  Specifically O(nnz(A) + n_col)
- * 
+ *
  */
 template <class I, class T>
 void csc_matvec(const I n_row,
-	            const I n_col, 
-	            const I Ap[], 
-	            const I Ai[], 
-	            const T Ax[],
-	            const T Xx[],
-	                  T Yx[])
-{ 
+                const I n_col,
+                const I Ap[],
+                const I Ai[],
+                const T Ax[],
+                const T Xx[],
+                      T Yx[])
+{
     for(I j = 0; j < n_col; j++){
         I col_start = Ap[j];
         I col_end   = Ap[j+1];
@@ -63,17 +63,17 @@ void csc_matvec(const I n_row,
  *
  * Note:
  *   Output array Yx must be preallocated
- * 
+ *
  */
 template <class I, class T>
 void csc_matvecs(const I n_row,
-	             const I n_col, 
+                 const I n_col,
                  const I n_vecs,
-	             const I Ap[], 
-	             const I Ai[], 
-	             const T Ax[],
-	             const T Xx[],
-	                   T Yx[])
+                 const I Ap[],
+                 const I Ai[],
+                 const T Ax[],
+                 const T Xx[],
+                       T Yx[])
 {
     for(I j = 0; j < n_col; j++){
         for(I ii = Ap[j]; ii < Ap[j+1]; ii++){
@@ -90,98 +90,99 @@ void csc_matvecs(const I n_row,
  * Derived methods
  */
 template <class I, class T>
-void csc_diagonal(const I n_row,
-                  const I n_col, 
-	              const I Ap[], 
-	              const I Aj[], 
-	              const T Ax[],
-	                    T Yx[])
-{ csr_diagonal(n_col, n_row, Ap, Aj, Ax, Yx); }
+void csc_diagonal(const I k,
+                  const I n_row,
+                  const I n_col,
+                  const I Ap[],
+                  const I Aj[],
+                  const T Ax[],
+                        T Yx[])
+{ csr_diagonal(-k, n_col, n_row, Ap, Aj, Ax, Yx); }
 
 
 template <class I, class T>
 void csc_tocsr(const I n_row,
-               const I n_col, 
-               const I Ap[], 
-               const I Ai[], 
+               const I n_col,
+               const I Ap[],
+               const I Ai[],
                const T Ax[],
                      I Bp[],
                      I Bj[],
                      T Bx[])
 { csr_tocsc<I,T>(n_col, n_row, Ap, Ai, Ax, Bp, Bj, Bx); }
 
-    
+
 template <class I>
 void csc_matmat_pass1(const I n_row,
-                      const I n_col, 
-                      const I Ap[], 
-                      const I Ai[], 
+                      const I n_col,
+                      const I Ap[],
+                      const I Ai[],
                       const I Bp[],
                       const I Bi[],
                             I Cp[])
 { csr_matmat_pass1(n_col, n_row, Bp, Bi, Ap, Ai, Cp); }
-    
+
 template <class I, class T>
 void csc_matmat_pass2(const I n_row,
-      	              const I n_col, 
-      	              const I Ap[], 
-      	              const I Ai[], 
-      	              const T Ax[],
-      	              const I Bp[],
-      	              const I Bi[],
-      	              const T Bx[],
-      	                    I Cp[],
-      	                    I Ci[],
-      	                    T Cx[])
+                      const I n_col,
+                      const I Ap[],
+                      const I Ai[],
+                      const T Ax[],
+                      const I Bp[],
+                      const I Bi[],
+                      const T Bx[],
+                            I Cp[],
+                            I Ci[],
+                            T Cx[])
 { csr_matmat_pass2(n_col, n_row, Bp, Bi, Bx, Ap, Ai, Ax, Cp, Ci, Cx); }
 
 template <class I, class T, class T2>
-void csc_ne_csc(const I n_row, const I n_col, 
-                   const I Ap[], const I Ai[], const T Ax[],
-                   const I Bp[], const I Bi[], const T Bx[],
-                         I Cp[],       I Ci[],      T2 Cx[])
+void csc_ne_csc(const I n_row, const I n_col,
+                const I Ap[], const I Ai[], const T Ax[],
+                const I Bp[], const I Bi[], const T Bx[],
+                      I Cp[],       I Ci[],      T2 Cx[])
 {
     csr_ne_csr(n_col, n_row, Ap, Ai, Ax, Bp, Bi, Bx, Cp, Ci, Cx);
 }
 
 template <class I, class T, class T2>
-void csc_lt_csc(const I n_row, const I n_col, 
-                   const I Ap[], const I Ai[], const T Ax[],
-                   const I Bp[], const I Bi[], const T Bx[],
-                         I Cp[],       I Ci[],      T2 Cx[])
+void csc_lt_csc(const I n_row, const I n_col,
+                const I Ap[], const I Ai[], const T Ax[],
+                const I Bp[], const I Bi[], const T Bx[],
+                      I Cp[],       I Ci[],      T2 Cx[])
 {
     csr_lt_csr(n_col, n_row, Ap, Ai, Ax, Bp, Bi, Bx, Cp, Ci, Cx);
 }
 
 template <class I, class T, class T2>
-void csc_gt_csc(const I n_row, const I n_col, 
-                   const I Ap[], const I Ai[], const T Ax[],
-                   const I Bp[], const I Bi[], const T Bx[],
-                         I Cp[],       I Ci[],      T2 Cx[])
+void csc_gt_csc(const I n_row, const I n_col,
+                const I Ap[], const I Ai[], const T Ax[],
+                const I Bp[], const I Bi[], const T Bx[],
+                      I Cp[],       I Ci[],      T2 Cx[])
 {
     csr_gt_csr(n_col, n_row, Ap, Ai, Ax, Bp, Bi, Bx, Cp, Ci, Cx);
 }
 
 template <class I, class T, class T2>
-void csc_le_csc(const I n_row, const I n_col, 
-                   const I Ap[], const I Ai[], const T Ax[],
-                   const I Bp[], const I Bi[], const T Bx[],
-                         I Cp[],       I Ci[],      T2 Cx[])
+void csc_le_csc(const I n_row, const I n_col,
+                const I Ap[], const I Ai[], const T Ax[],
+                const I Bp[], const I Bi[], const T Bx[],
+                      I Cp[],       I Ci[],      T2 Cx[])
 {
     csr_le_csr(n_col, n_row, Ap, Ai, Ax, Bp, Bi, Bx, Cp, Ci, Cx);
 }
 
 template <class I, class T, class T2>
-void csc_ge_csc(const I n_row, const I n_col, 
-                   const I Ap[], const I Ai[], const T Ax[],
-                   const I Bp[], const I Bi[], const T Bx[],
-                         I Cp[],       I Ci[],      T2 Cx[])
+void csc_ge_csc(const I n_row, const I n_col,
+                const I Ap[], const I Ai[], const T Ax[],
+                const I Bp[], const I Bi[], const T Bx[],
+                      I Cp[],       I Ci[],      T2 Cx[])
 {
     csr_ge_csr(n_col, n_row, Ap, Ai, Ax, Bp, Bi, Bx, Cp, Ci, Cx);
 }
 
 template <class I, class T>
-void csc_elmul_csc(const I n_row, const I n_col, 
+void csc_elmul_csc(const I n_row, const I n_col,
                    const I Ap[], const I Ai[], const T Ax[],
                    const I Bp[], const I Bi[], const T Bx[],
                          I Cp[],       I Ci[],       T Cx[])
@@ -190,7 +191,7 @@ void csc_elmul_csc(const I n_row, const I n_col,
 }
 
 template <class I, class T>
-void csc_eldiv_csc(const I n_row, const I n_col, 
+void csc_eldiv_csc(const I n_row, const I n_col,
                    const I Ap[], const I Ai[], const T Ax[],
                    const I Bp[], const I Bi[], const T Bx[],
                          I Cp[],       I Ci[],       T Cx[])
@@ -200,7 +201,7 @@ void csc_eldiv_csc(const I n_row, const I n_col,
 
 
 template <class I, class T>
-void csc_plus_csc(const I n_row, const I n_col, 
+void csc_plus_csc(const I n_row, const I n_col,
                   const I Ap[], const I Ai[], const T Ax[],
                   const I Bp[], const I Bi[], const T Bx[],
                         I Cp[],       I Ci[],       T Cx[])
@@ -209,7 +210,7 @@ void csc_plus_csc(const I n_row, const I n_col,
 }
 
 template <class I, class T>
-void csc_minus_csc(const I n_row, const I n_col, 
+void csc_minus_csc(const I n_row, const I n_col,
                    const I Ap[], const I Ai[], const T Ax[],
                    const I Bp[], const I Bi[], const T Bx[],
                          I Cp[],       I Ci[],       T Cx[])
@@ -219,19 +220,19 @@ void csc_minus_csc(const I n_row, const I n_col,
 
 
 template <class I, class T>
-void csc_maximum_csc(const I n_row, const I n_col, 
-                  const I Ap[], const I Ai[], const T Ax[],
-                  const I Bp[], const I Bi[], const T Bx[],
-                        I Cp[],       I Ci[],       T Cx[])
+void csc_maximum_csc(const I n_row, const I n_col,
+                     const I Ap[], const I Ai[], const T Ax[],
+                     const I Bp[], const I Bi[], const T Bx[],
+                           I Cp[],       I Ci[],       T Cx[])
 {
     csr_maximum_csr(n_col, n_row, Ap, Ai, Ax, Bp, Bi, Bx, Cp, Ci, Cx);
 }
 
 template <class I, class T>
-void csc_minimum_csc(const I n_row, const I n_col, 
-                   const I Ap[], const I Ai[], const T Ax[],
-                   const I Bp[], const I Bi[], const T Bx[],
-                         I Cp[],       I Ci[],       T Cx[])
+void csc_minimum_csc(const I n_row, const I n_col,
+                     const I Ap[], const I Ai[], const T Ax[],
+                     const I Bp[], const I Bi[], const T Bx[],
+                           I Cp[],       I Ci[],       T Cx[])
 {
     csr_minimum_csr(n_col, n_row, Ap, Ai, Ax, Bp, Bi, Bx, Cp, Ci, Cx);
 }
