@@ -4,8 +4,11 @@ from __future__ import division, print_function, absolute_import
 
 import numpy as np
 from numpy import array, matrix
-from numpy.testing import TestCase, run_module_suite, assert_equal, \
-        assert_array_equal, assert_raises, assert_array_almost_equal_nulp
+from numpy.testing import (assert_equal, assert_,
+        assert_array_equal, assert_array_almost_equal_nulp)
+import pytest
+from pytest import raises as assert_raises
+from scipy._lib._testutils import check_free_memory
 
 from scipy.sparse import csr_matrix, coo_matrix
 
@@ -17,7 +20,18 @@ sparse_formats = ['csr','csc','coo','bsr','dia','lil','dok']
 #TODO check whether format=XXX is respected
 
 
-class TestConstructUtils(TestCase):
+def _sprandn(m, n, density=0.01, format="coo", dtype=None, random_state=None):
+    # Helper function for testing.
+    if random_state is None:
+        random_state = np.random
+    elif isinstance(random_state, (int, np.integer)):
+        random_state = np.random.RandomState(random_state)
+    data_rvs = random_state.randn
+    return construct.random(m, n, density, format, dtype,
+                            random_state, data_rvs)
+
+
+class TestConstructUtils(object):
     def test_spdiags(self):
         diags1 = array([[1, 2, 3, 4, 5]])
         diags2 = array([[1, 2, 3, 4, 5],
@@ -80,7 +94,20 @@ class TestConstructUtils(TestCase):
         cases.append(([a[:1]],[-1], (2, 2), [[0,0],[1,0]]))
         cases.append(([a[:3]], [0], (3, 4), [[1,0,0,0],[0,2,0,0],[0,0,3,0]]))
         cases.append(([a[:3]], [1], (3, 4), [[0,1,0,0],[0,0,2,0],[0,0,0,3]]))
+        cases.append(([a[:1]], [-2], (3, 5), [[0,0,0,0,0],[0,0,0,0,0],[1,0,0,0,0]]))
+        cases.append(([a[:2]], [-1], (3, 5), [[0,0,0,0,0],[1,0,0,0,0],[0,2,0,0,0]]))
+        cases.append(([a[:3]], [0], (3, 5), [[1,0,0,0,0],[0,2,0,0,0],[0,0,3,0,0]]))
+        cases.append(([a[:3]], [1], (3, 5), [[0,1,0,0,0],[0,0,2,0,0],[0,0,0,3,0]]))
         cases.append(([a[:3]], [2], (3, 5), [[0,0,1,0,0],[0,0,0,2,0],[0,0,0,0,3]]))
+        cases.append(([a[:2]], [3], (3, 5), [[0,0,0,1,0],[0,0,0,0,2],[0,0,0,0,0]]))
+        cases.append(([a[:1]], [4], (3, 5), [[0,0,0,0,1],[0,0,0,0,0],[0,0,0,0,0]]))
+        cases.append(([a[:1]], [-4], (5, 3), [[0,0,0],[0,0,0],[0,0,0],[0,0,0],[1,0,0]]))
+        cases.append(([a[:2]], [-3], (5, 3), [[0,0,0],[0,0,0],[0,0,0],[1,0,0],[0,2,0]]))
+        cases.append(([a[:3]], [-2], (5, 3), [[0,0,0],[0,0,0],[1,0,0],[0,2,0],[0,0,3]]))
+        cases.append(([a[:3]], [-1], (5, 3), [[0,0,0],[1,0,0],[0,2,0],[0,0,3],[0,0,0]]))
+        cases.append(([a[:3]], [0], (5, 3), [[1,0,0],[0,2,0],[0,0,3],[0,0,0],[0,0,0]]))
+        cases.append(([a[:2]], [1], (5, 3), [[0,1,0],[0,0,2],[0,0,0],[0,0,0],[0,0,0]]))
+        cases.append(([a[:1]], [2], (5, 3), [[0,0,1],[0,0,0],[0,0,0],[0,0,0],[0,0,0]]))
 
         cases.append(([a[:3],b[:1]], [0,2], (3, 3), [[1,0,6],[0,2,0],[0,0,3]]))
         cases.append(([a[:2],b[:3]], [-1,0], (3, 4), [[6,0,0,0],[1,7,0,0],[0,2,8,0]]))
@@ -103,6 +130,11 @@ class TestConstructUtils(TestCase):
                                                           [1, 0, 0,14, 0],
                                                           [0, 2, 0, 0,15]]))
 
+        # too long arrays are OK
+        cases.append(([a], [0], (1, 1), [[1]]))
+        cases.append(([a[:3],b], [0,2], (3, 3), [[1, 0, 6], [0, 2, 0], [0, 0, 3]]))
+        cases.append((np.array([[1, 2, 3], [4, 5, 6]]), [0,-1], (3, 3), [[1, 0, 0], [4, 2, 0], [0, 5, 3]]))
+
         # scalar case: broadcasting
         cases.append(([1,-2,1], [1,0,-1], (3, 3), [[-2, 1, 0],
                                                     [1, -2, 1],
@@ -113,13 +145,21 @@ class TestConstructUtils(TestCase):
                 assert_equal(construct.diags(d, o, shape=shape).todense(),
                              result)
 
-                if shape[0] == shape[1] and hasattr(d[0], '__len__'):
+                if shape[0] == shape[1] and hasattr(d[0], '__len__') and len(d[0]) <= max(shape):
                     # should be able to find the shape automatically
                     assert_equal(construct.diags(d, o).todense(), result)
             except:
-                print("%r %r %r" % (d, o, shape))
+                print("%r %r %r %r" % (d, o, shape, result))
                 raise
 
+    def test_diags_default(self):
+        a = array([1, 2, 3, 4, 5])
+        assert_equal(construct.diags(a).todense(), np.diag(a))
+
+    def test_diags_default_bad(self):
+        a = array([[1, 2, 3, 4, 5], [2, 3, 4, 5, 6]])
+        assert_raises(ValueError, construct.diags, a)
+
     def test_diags_bad(self):
         a = array([1, 2, 3, 4, 5])
         b = array([6, 7, 8, 9, 10])
@@ -127,14 +167,11 @@ class TestConstructUtils(TestCase):
 
         cases = []
         cases.append(([a[:0]], 0, (1, 1)))
-        cases.append(([a], [0], (1, 1)))
-        cases.append(([a[:3],b], [0,2], (3, 3)))
         cases.append(([a[:4],b,c[:3]], [-1,0,1], (5, 5)))
         cases.append(([a[:2],c,b[:3]], [-4,2,-1], (6, 5)))
         cases.append(([a[:2],c,b[:3]], [-4,2,-1], None))
         cases.append(([], [-4,2,-1], None))
-        cases.append(([1], [-4], (4, 4)))
-        cases.append(([a[:0]], [-1], (1, 2)))
+        cases.append(([1], [-5], (4, 4)))
         cases.append(([a], 0, None))
 
         for d, o, shape in cases:
@@ -184,6 +221,10 @@ class TestConstructUtils(TestCase):
             assert_equal(construct.diags(d, k).toarray(),
                          construct.diags([d], [k]).toarray())
 
+    def test_diags_empty(self):
+        x = construct.diags([])
+        assert_equal(x.shape, (0, 0))
+
     def test_identity(self):
         assert_equal(construct.identity(1).toarray(), [[1]])
         assert_equal(construct.identity(2).toarray(), [[1,0],[0,1]])
@@ -281,6 +322,10 @@ class TestConstructUtils(TestCase):
                      expected)
         assert_equal(construct.vstack([A.tocsr(),B.tocsr()], dtype=np.float32).dtype,
                      np.float32)
+        assert_equal(construct.vstack([A.tocsr(),B.tocsr()],
+                                      dtype=np.float32).indices.dtype, np.int32)
+        assert_equal(construct.vstack([A.tocsr(),B.tocsr()],
+                                      dtype=np.float32).indptr.dtype, np.int32)
 
     def test_hstack(self):
 
@@ -301,6 +346,7 @@ class TestConstructUtils(TestCase):
         A = coo_matrix([[1,2],[3,4]])
         B = coo_matrix([[5],[6]])
         C = coo_matrix([[7]])
+        D = coo_matrix((0,0))
 
         expected = matrix([[1, 2, 5],
                            [3, 4, 6],
@@ -317,7 +363,36 @@ class TestConstructUtils(TestCase):
                            [7, 0]])
         assert_equal(construct.bmat([[None,B],[C,None]]).todense(), expected)
 
-        #TODO test failure cases
+        expected = matrix(np.empty((0,0)))
+        assert_equal(construct.bmat([[None,None]]).todense(), expected)
+        assert_equal(construct.bmat([[None,D],[D,None]]).todense(), expected)
+
+        # test bug reported in gh-5976
+        expected = matrix([[7]])
+        assert_equal(construct.bmat([[None,D],[C,None]]).todense(), expected)
+
+        # test failure cases
+        with assert_raises(ValueError) as excinfo:
+            construct.bmat([[A], [B]])
+        excinfo.match(r'Got blocks\[1,0\]\.shape\[1\] == 1, expected 2')
+
+        with assert_raises(ValueError) as excinfo:
+            construct.bmat([[A, C]])
+        excinfo.match(r'Got blocks\[0,1\]\.shape\[0\] == 1, expected 2')
+
+    @pytest.mark.slow
+    def test_concatenate_int32_overflow(self):
+        """ test for indptr overflow when concatenating matrices """
+        check_free_memory(30000)
+        
+        n = 33000
+        A = csr_matrix(np.ones((n, n), dtype=bool))
+        B = A.copy()
+        C = construct._compressed_sparse_stack((A,B), 0)
+        
+        assert_(np.all(np.equal(np.diff(C.indptr), n)))
+        assert_equal(C.indices.dtype, np.int64)
+        assert_equal(C.indptr.dtype, np.int64)
 
     def test_block_diag_basic(self):
         """ basic test for block_diag """
@@ -351,32 +426,55 @@ class TestConstructUtils(TestCase):
         assert_equal(construct.block_diag([1]).todense(),
                      matrix([[1]]))
 
+    def test_random_sampling(self):
+        # Simple sanity checks for sparse random sampling.
+        for f in sprand, _sprandn:
+            for t in [np.float32, np.float64, np.longdouble]:
+                x = f(5, 10, density=0.1, dtype=t)
+                assert_equal(x.dtype, t)
+                assert_equal(x.shape, (5, 10))
+                assert_equal(x.nonzero()[0].size, 5)
+
+            x1 = f(5, 10, density=0.1, random_state=4321)
+            assert_equal(x1.dtype, np.double)
+
+            x2 = f(5, 10, density=0.1, random_state=np.random.RandomState(4321))
+
+            assert_array_equal(x1.data, x2.data)
+            assert_array_equal(x1.row, x2.row)
+            assert_array_equal(x1.col, x2.col)
+
+            for density in [0.0, 0.1, 0.5, 1.0]:
+                x = f(5, 10, density=density)
+                assert_equal(x.nnz, int(density * np.prod(x.shape)))
+
+            for fmt in ['coo', 'csc', 'csr', 'lil']:
+                x = f(5, 10, format=fmt)
+                assert_equal(x.format, fmt)
+
+            assert_raises(ValueError, lambda: f(5, 10, 1.1))
+            assert_raises(ValueError, lambda: f(5, 10, -0.1))
+
     def test_rand(self):
-        # Simple sanity checks for sparse.rand
-        for t in [np.float32, np.float64, np.longdouble]:
-            x = sprand(5, 10, density=0.1, dtype=t)
-            assert_equal(x.dtype, t)
-            assert_equal(x.shape, (5, 10))
-            assert_equal(x.nonzero()[0].size, 5)
+        # Simple distributional checks for sparse.rand.
+        for random_state in None, 4321, np.random.RandomState():
+            x = sprand(10, 20, density=0.5, dtype=np.float64,
+                       random_state=random_state)
+            assert_(np.all(np.less_equal(0, x.data)))
+            assert_(np.all(np.less_equal(x.data, 1)))
 
-        x1 = sprand(5, 10, density=0.1,
-                    random_state=4321)
-        assert_equal(x1.dtype, np.double)
+    def test_randn(self):
+        # Simple distributional checks for sparse.randn.
+        # Statistically, some of these should be negative
+        # and some should be greater than 1.
+        for random_state in None, 4321, np.random.RandomState():
+            x = _sprandn(10, 20, density=0.5, dtype=np.float64,
+                         random_state=random_state)
+            assert_(np.any(np.less(x.data, 0)))
+            assert_(np.any(np.less(1, x.data)))
 
-        x2 = sprand(5, 10, density=0.1,
-                    random_state=np.random.RandomState(4321))
+    def test_random_accept_str_dtype(self):
+        # anything that np.dtype can convert to a dtype should be accepted
+        # for the dtype
+        a = construct.random(10, 10, dtype='d')
 
-        assert_array_equal(x1.data, x2.data)
-        assert_array_equal(x1.row, x2.row)
-        assert_array_equal(x1.col, x2.col)
-
-        for fmt in ['coo', 'csc', 'csr', 'lil']:
-            x = sprand(5, 10, format=fmt)
-            assert_equal(x.format, fmt)
-
-        assert_raises(ValueError, lambda: sprand(5, 10, 1.1))
-        assert_raises(ValueError, lambda: sprand(5, 10, -0.1))
-
-
-if __name__ == "__main__":
-    run_module_suite()
