@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-""" cythonize
+"""cythonize
 
 Cythonize pyx files into C files as needed.
 
@@ -7,6 +7,10 @@ Usage: cythonize [root_dir]
 
 Default [root_dir] is 'scipy'.
 
+The number of parallel Cython processes is controlled by the
+environment variable SCIPY_NUM_CYTHONIZE_JOBS. If not set, determined
+from the number of CPUs.
+
 Checks pyx files to see if they have been changed relative to their
 corresponding C files.  If they have, then runs cython on these files to
 recreate the C files.
@@ -28,6 +32,7 @@ https://raw.github.com/dagss/private-scipy-refactor/cythonize/cythonize.py
 
 Note: this script does not check any of the dependent C libraries; it only
 operates on the Cython .pyx files.
+
 """
 
 from __future__ import division, print_function, absolute_import
@@ -37,6 +42,8 @@ import re
 import sys
 import hashlib
 import subprocess
+from multiprocessing.dummy import Pool, Lock
+from os.path import dirname, join
 
 HASH_FILE = 'cythonize.dat'
 DEFAULT_ROOT = 'scipy'
@@ -50,12 +57,29 @@ except NameError:
 #
 # Rules
 #
-def process_pyx(fromfile, tofile):
+def process_pyx(fromfile, tofile, cwd):
     try:
         from Cython.Compiler.Version import version as cython_version
         from distutils.version import LooseVersion
-        if LooseVersion(cython_version) < LooseVersion('0.23.4'):
-            raise Exception('Building SciPy requires Cython >= 0.23.4')
+
+        # Try to find pyproject.toml
+        pyproject_toml = join(dirname(__file__), '..', 'pyproject.toml')
+        if not os.path.exists(pyproject_toml):
+            raise ImportError()
+
+        # Try to find the minimum version from pyproject.toml
+        with open(pyproject_toml) as pt:
+            for line in pt:
+                if "cython" not in line.lower():
+                    continue
+                _, line = line.split('=')
+                required_version, _ = line.split('"')
+                break
+            else:
+                raise ImportError()
+
+        if LooseVersion(cython_version) < LooseVersion(required_version):
+            raise Exception('Building SciPy requires Cython >= ' + required_version)
 
     except ImportError:
         pass
@@ -66,7 +90,7 @@ def process_pyx(fromfile, tofile):
 
     try:
         try:
-            r = subprocess.call(['cython'] + flags + ["-o", tofile, fromfile])
+            r = subprocess.call(['cython'] + flags + ["-o", tofile, fromfile], cwd=cwd)
             if r != 0:
                 raise Exception('Cython failed')
         except OSError:
@@ -75,13 +99,14 @@ def process_pyx(fromfile, tofile):
             r = subprocess.call([sys.executable, '-c',
                                  'import sys; from Cython.Compiler.Main import '
                                  'setuptools_main as main; sys.exit(main())'] + flags +
-                                 ["-o", tofile, fromfile])
+                                 ["-o", tofile, fromfile],
+                                cwd=cwd)
             if r != 0:
                 raise Exception("Cython either isn't installed or it failed.")
     except OSError:
         raise OSError('Cython needs to be installed')
 
-def process_tempita_pyx(fromfile, tofile):
+def process_tempita_pyx(fromfile, tofile, cwd):
     try:
         try:
             from Cython import Tempita as tempita
@@ -91,19 +116,22 @@ def process_tempita_pyx(fromfile, tofile):
         raise Exception('Building SciPy requires Tempita: '
                         'pip install --user Tempita')
     from_filename = tempita.Template.from_filename
-    template = from_filename(fromfile, encoding=sys.getdefaultencoding())
+    template = from_filename(os.path.join(cwd, fromfile),
+                             encoding=sys.getdefaultencoding())
     pyxcontent = template.substitute()
     assert fromfile.endswith('.pyx.in')
     pyxfile = fromfile[:-len('.pyx.in')] + '.pyx'
-    with open(pyxfile, "w") as f:
+    with open(os.path.join(cwd, pyxfile), "w") as f:
         f.write(pyxcontent)
-    process_pyx(pyxfile, tofile)
+    process_pyx(pyxfile, tofile, cwd)
+
 
 rules = {
     # fromext : function
     '.pyx': process_pyx,
     '.pyx.in': process_tempita_pyx
     }
+
 #
 # Hash db
 #
@@ -150,58 +178,98 @@ def get_hash(frompath, topath):
         to_hash = None
     return (from_hash, to_hash)
 
-def get_pxi_dependencies(fullfrompath):
+def get_cython_dependencies(fullfrompath):
     fullfromdir = os.path.dirname(fullfrompath)
-    dependencies = []
+    deps = set()
     with open(fullfrompath, 'r') as f:
+        pxipattern = re.compile('include "([a-zA-Z0-9_]+\.pxi)"')
+        pxdpattern1 = re.compile('from \. cimport ([a-zA-Z0-9_]+)')
+        pxdpattern2 = re.compile('from \.([a-zA-Z0-9_]+) cimport')
+
         for line in f:
-            line = [token.strip('\'\" \r\n') for token in line.split(' ')]
-            if line[0] == "include":
-                dependencies.append(os.path.join(fullfromdir, line[1]))
-    return dependencies
+            m = pxipattern.match(line)
+            if m:
+                deps.add(os.path.join(fullfromdir, m.group(1)))
+            m = pxdpattern1.match(line)
+            if m:
+                deps.add(os.path.join(fullfromdir, m.group(1) + '.pxd'))
+            m = pxdpattern2.match(line)
+            if m:
+                deps.add(os.path.join(fullfromdir, m.group(1) + '.pxd'))
+    return list(deps)
 
-def process(path, fromfile, tofile, processor_function, hash_db, pxi_hashes):
-    fullfrompath = os.path.join(path, fromfile)
-    fulltopath = os.path.join(path, tofile)
-    current_hash = get_hash(fullfrompath, fulltopath)
-    if current_hash == hash_db.get(normpath(fullfrompath), None):
-        file_changed = False
-    else:
-        file_changed = True
-
-    pxi_changed = False
-    pxi_dependencies = get_pxi_dependencies(fullfrompath)
-    for pxi in pxi_dependencies:
-        pxi_hash = get_hash(pxi, None)
-        if pxi_hash == hash_db.get(normpath(pxi), None):
-            continue
+def process(path, fromfile, tofile, processor_function, hash_db,
+            dep_hashes, lock):
+    with lock:
+        fullfrompath = os.path.join(path, fromfile)
+        fulltopath = os.path.join(path, tofile)
+        current_hash = get_hash(fullfrompath, fulltopath)
+        if current_hash == hash_db.get(normpath(fullfrompath), None):
+            file_changed = False
         else:
-            pxi_hashes[normpath(pxi)] = pxi_hash
-            pxi_changed = True
+            file_changed = True
 
-    if not file_changed and not pxi_changed:
-        print('%s has not changed' % fullfrompath)
-        return
+        deps_changed = False
+        deps = get_cython_dependencies(fullfrompath)
+        for dep in deps:
+            dep_hash = get_hash(dep, None)
+            if dep_hash == hash_db.get(normpath(dep), None):
+                continue
+            else:
+                dep_hashes[normpath(dep)] = dep_hash
+                deps_changed = True
+
+        if not file_changed and not deps_changed:
+            print('%s has not changed' % fullfrompath)
+            sys.stdout.flush()
+            return
 
-    orig_cwd = os.getcwd()
-    try:
-        os.chdir(path)
         print('Processing %s' % fullfrompath)
-        processor_function(fromfile, tofile)
-    finally:
-        os.chdir(orig_cwd)
-    # changed target file, recompute hash
-    current_hash = get_hash(fullfrompath, fulltopath)
-    # store hash in db
-    hash_db[normpath(fullfrompath)] = current_hash
+        sys.stdout.flush()
 
+    processor_function(fromfile, tofile, cwd=path)
+
+    with lock:
+        # changed target file, recompute hash
+        current_hash = get_hash(fullfrompath, fulltopath)
+        # store hash in db
+        hash_db[normpath(fullfrompath)] = current_hash
+
+def process_generate_pyx(path, lock):
+    with lock:
+        print('Running {}'.format(path))
+    ret = subprocess.call([sys.executable, path])
+    with lock:
+        if ret != 0:
+            raise RuntimeError("Running {} failed".format(path))
 
 def find_process_files(root_dir):
+    lock = Lock()
+
+    try:
+        num_proc = int(os.environ.get('SCIPY_NUM_CYTHONIZE_JOBS', ''))
+        pool = Pool(processes=num_proc)
+    except ValueError:
+        pool = Pool()
+
     hash_db = load_hashes(HASH_FILE)
-    # Keep changed .pxi hashes in a separate dict until the end
+    # Keep changed pxi/pxd hashes in a separate dict until the end
     # because if we update hash_db and multiple files include the same
     # .pxi file the changes won't be detected.
-    pxi_hashes = {}
+    dep_hashes = {}
+
+    # Run any _generate_pyx.py scripts
+    jobs = []
+    for cur_dir, dirs, files in os.walk(root_dir):
+        generate_pyx = os.path.join(cur_dir, '_generate_pyx.py')
+        if os.path.exists(generate_pyx):
+            jobs.append(generate_pyx)
+
+    for result in pool.imap_unordered(lambda fn: process_generate_pyx(fn, lock), jobs):
+        pass
+
+    # Process pyx files
+    jobs = []
     for cur_dir, dirs, files in os.walk(root_dir):
         for filename in files:
             in_file = os.path.join(cur_dir, filename + ".in")
@@ -217,8 +285,13 @@ def find_process_files(root_dir):
                             toext = ".cxx"
                     fromfile = filename
                     tofile = filename[:-len(fromext)] + toext
-                    process(cur_dir, fromfile, tofile, function, hash_db, pxi_hashes)
-    hash_db.update(pxi_hashes)
+                    jobs.append((cur_dir, fromfile, tofile, function,
+                                 hash_db, dep_hashes, lock))
+
+    for result in pool.imap_unordered(lambda args: process(*args), jobs):
+        pass
+
+    hash_db.update(dep_hashes)
     save_hashes(hash_db, HASH_FILE)
 
 def main():
